@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -24,20 +24,42 @@
 #include "swarm.h"
 #include "db.h"
 #include "qgram.h"
-#include "matrix.h"
 #include "nw.h"
 #include "scan.h"
 #include "util.h"
+#include "utils/score_matrix.h"
+#include <algorithm>  // std::min()
+#include <cinttypes>  // macros PRIu64 and PRId64
+#include <cstdint>  // int64_t, uint64_t
+#include <cstdio>  // fputc()
+#include <cstdlib>  // qsort()
+#include <cstring>  // strcmp
+#include <iterator> // next
+#include <vector>
+
+#ifndef PRIu64
+#ifdef _WIN32
+#define PRIu64 "I64u"
+#else
+constexpr char PRIu64[] = "lu";
+#endif
+#endif
+
+#ifndef PRId64
+#ifdef _WIN32
+#define PRId64 "I64d"
+#else
+constexpr char PRId64[] = "ld";
+#endif
+#endif
+
 
 static uint64_t count_comparisons_8;
 static uint64_t count_comparisons_16;
 
 static uint64_t targetcount;
-static uint64_t * targetindices;
-static uint64_t * targetampliconids;
 static uint64_t * scores;
 static uint64_t * diffs;
-static uint64_t * alignlengths;
 static uint64_t * qgramamps;
 static uint64_t * qgramdiffs;
 static uint64_t * qgramindices;
@@ -56,53 +78,28 @@ static uint64_t seeded;
 
 struct swarminfo_t
 {
-  uint64_t mass;
-  unsigned int seed;
-  int dummy; /* alignment padding only */
+  uint64_t mass {0};
+  unsigned int seed {0};
+  int dummy {0}; /* alignment padding only */
 };
 
-auto compare_mass_seed(const void * a, const void * b) -> int;
 
-auto compare_mass_seed(const void * a, const void * b) -> int
-{
-  const auto * x = static_cast<const struct swarminfo_t *>(a);
-  const auto * y = static_cast<const struct swarminfo_t *>(b);
-
-  uint64_t m = x->mass;
-  uint64_t n = y->mass;
-  int status {0};
-
-  if (m > n) {
-    status = -1;
-  }
-  else if (m < n) {
-    status = +1;
-  }
-  else {
-    status = strcmp(db_getheader(x->seed), db_getheader(y->seed));
-  }
-  return status;
-}
-
-
-auto write_representative_sequences(const uint64_t amplicons,
-                                    struct Parameters const & p) -> void {
-
-  uint64_t swarmcount {0};
-  progress_init("Sorting seeds:    ", amplicons);
-  auto * swarminfo = new struct swarminfo_t[swarmed];
-  uint64_t mass {0};
-  unsigned previd = amps[0].swarmid;
-  unsigned seed = amps[0].ampliconid;
+auto collect_seeds(const uint64_t amplicons) -> std::vector<struct swarminfo_t> {
+  progress_init("Collecting seeds:    ", amplicons);
+  std::vector<struct swarminfo_t> seeds(swarmed);  // swarmed == amplicons! Discard swarmed?
+  auto swarmcount = 0L;
+  uint64_t mass = 0;
+  unsigned int previd = amps[0].swarmid;
+  unsigned int seed = amps[0].ampliconid;
   mass += db_getabundance(seed);
   for(auto i = 1ULL; i < amplicons; i++)
     {
-      unsigned id = amps[i].swarmid;
+      const unsigned int id = amps[i].swarmid;
       if (id != previd)
         {
-          swarminfo[swarmcount].seed = seed;
-          swarminfo[swarmcount].mass = mass;
-          swarmcount++;
+          seeds[swarmcount].seed = seed;  // update previous
+          seeds[swarmcount].mass = mass;
+          ++swarmcount;
           mass = 0;
           seed = amps[i].ampliconid;
         }
@@ -110,125 +107,160 @@ auto write_representative_sequences(const uint64_t amplicons,
       previd = id;
       progress_update(i);
     }
-  swarminfo[swarmcount].seed = seed;
-  swarminfo[swarmcount].mass = mass;
-  swarmcount++;
-  qsort(swarminfo, swarmcount, sizeof(swarminfo_t), compare_mass_seed);
-  progress_done();
+  seeds[swarmcount].seed = seed;
+  seeds[swarmcount].mass = mass;
+  ++swarmcount;
 
-  progress_init("Writing seeds:    ", swarmcount);
-  for(auto i = 0ULL; i < swarmcount; i++)
-    {
-      uint64_t swarm_mass = swarminfo[i].mass;
-      unsigned int swarm_seed = swarminfo[i].seed;
+  // free some memory
+  seeds.erase(std::next(seeds.begin(), swarmcount), seeds.end());
+  seeds.shrink_to_fit();
 
-      fprintf(fp_seeds, ">");
-      fprint_id_with_new_abundance(fp_seeds, swarm_seed, swarm_mass, p.opt_usearch_abundance);
-      fprintf(fp_seeds, "\n");
-      db_fprintseq(fp_seeds, swarm_seed);
-      progress_update(i);
+  return seeds;
+}
+
+
+auto sort_seeds(std::vector<struct swarminfo_t>& seeds) -> void {
+  progress_init("Sorting seeds:    ", seeds.size());
+
+  auto compare_seeds = [](struct swarminfo_t const& lhs, struct swarminfo_t const& rhs) -> bool {
+    // sort by decreasing mass...
+    if (lhs.mass > rhs.mass) {
+      return true;
     }
-  delete [] swarminfo;
-  swarminfo = nullptr;
+    if (lhs.mass < rhs.mass) {
+      return false;
+    }
+    // ...then ties are sorted by label (alphabetical order)
+    auto * const lhs_header = db_getheader(lhs.seed);
+    auto * const rhs_header = db_getheader(rhs.seed);
+    const auto results = std::strcmp(lhs_header, rhs_header);
+    return results == -1;
+  };
+
+  std::sort(seeds.begin(), seeds.end(), compare_seeds);
   progress_done();
 }
 
 
+auto write_seeds(std::vector<struct swarminfo_t> const & seeds,
+                 bool const opt_usearch_abundance) -> void {
+  progress_init("Writing seeds:    ", seeds.size());
+  auto ticker = 0ULL;  // refactoring: C++20 move ticker to range-loop init-statement
+  for(const auto& seed: seeds) {
+      const auto swarm_mass = seed.mass;
+      const auto swarm_seed = seed.seed;
+
+      std::fprintf(fp_seeds, ">");
+      fprint_id_with_new_abundance(fp_seeds, swarm_seed, swarm_mass, opt_usearch_abundance);
+      std::fprintf(fp_seeds, "\n");
+      db_fprintseq(fp_seeds, swarm_seed);
+      progress_update(ticker);
+      ++ticker;
+  }
+  progress_done();
+}
+
+
+auto write_representative_sequences(const uint64_t amplicons,
+                                    struct Parameters const & parameters) -> void {
+  auto seeds = collect_seeds(amplicons);
+  sort_seeds(seeds);
+  write_seeds(seeds, parameters.opt_usearch_abundance);
+}
+
+
 auto write_swarms_default_format(const uint64_t amplicons,
-                                 struct Parameters const & p) -> void {
+                                 struct Parameters const & parameters) -> void {
   /* native swarm output */
-  constexpr char sep_amplicons {sepchar};  /* usually a space */
-  constexpr char sep_swarms {'\n'};
+  static constexpr char sep_amplicons {sepchar};  /* usually a space */
+  static constexpr char sep_swarms {'\n'};
 
   fprint_id(outfile, amps[0].ampliconid,
-            p.opt_usearch_abundance, p.opt_append_abundance);
+            parameters.opt_usearch_abundance, parameters.opt_append_abundance);
   int64_t previd = amps[0].swarmid;
 
   for(auto i = 1ULL; i < amplicons; i++)
     {
-      int64_t id = amps[i].swarmid;
+      const int64_t id = amps[i].swarmid;
       if (id == previd) {
-        fputc(sep_amplicons, outfile);
+        std::fputc(sep_amplicons, outfile);
       }
       else {
-        fputc(sep_swarms, outfile);
+        std::fputc(sep_swarms, outfile);
       }
       fprint_id(outfile, amps[i].ampliconid,
-                p.opt_usearch_abundance, p.opt_append_abundance);
+                parameters.opt_usearch_abundance, parameters.opt_append_abundance);
       previd = id;
     }
-  fputc('\n', outfile);
+  std::fputc('\n', outfile);
 }
 
 
 auto write_swarms_mothur_format(const uint64_t amplicons,
                                 const unsigned int swarmid,
-                                struct Parameters const & p) -> void {
+                                struct Parameters const & parameters) -> void {
   /* mothur list file output */
-  constexpr char sep_amplicons {','};
-  constexpr char sep_swarms {'\t'};
+  static constexpr char sep_amplicons {','};
+  static constexpr char sep_swarms {'\t'};
 
-  fprintf(outfile, "swarm_%" PRId64 "\t%u\t", p.opt_differences, swarmid);
+  std::fprintf(outfile, "swarm_%" PRId64 "\t%u\t", parameters.opt_differences, swarmid);
 
   fprint_id(outfile, amps[0].ampliconid,
-            p.opt_usearch_abundance, p.opt_append_abundance);
+            parameters.opt_usearch_abundance, parameters.opt_append_abundance);
   int64_t previd = amps[0].swarmid;
 
   for(auto i = 1ULL; i < amplicons; i++)
     {
-      int64_t id = amps[i].swarmid;
+      const int64_t id = amps[i].swarmid;
       if (id == previd) {
-        fputc(sep_amplicons, outfile);
+        std::fputc(sep_amplicons, outfile);
       }
       else {
-        fputc(sep_swarms, outfile);
+        std::fputc(sep_swarms, outfile);
       }
       fprint_id(outfile, amps[i].ampliconid,
-                p.opt_usearch_abundance, p.opt_append_abundance);
+                parameters.opt_usearch_abundance, parameters.opt_append_abundance);
       previd = id;
     }
 
-  fputc('\n', outfile);
+  std::fputc('\n', outfile);
 }
 
 
-void algo_run(struct Parameters const & p)
+auto algo_run(struct Parameters const & parameters) -> void
 {
-  score_matrix_read(p);
+  const auto score_matrix_63 = create_score_matrix<int64_t>(parameters.penalty_mismatch);
+
   search_begin();
 
   count_comparisons_8 = 0;
   count_comparisons_16 = 0;
 
-#ifdef VERBOSE
-  uint64_t searches {0};
-  uint64_t estimates {0};
-#endif
 
   uint64_t largestswarm {0};
 
   uint64_t maxgenerations {0};
 
-  uint64_t amplicons = db_getsequencecount();
-  uint64_t longestamplicon = db_getlongestsequence();
+  const uint64_t amplicons = db_getsequencecount();
+  const uint64_t longestamplicon = db_getlongestsequence();
 
   db_qgrams_init();
 
   qgram_diff_init();
 
   amps = new struct ampliconinfo_s[amplicons];
-  targetampliconids = new uint64_t[amplicons];
-  targetindices = new uint64_t[amplicons];
+  std::vector<uint64_t> targetampliconids(amplicons);
+  std::vector<uint64_t> targetindices(amplicons);
   scores = new uint64_t[amplicons];
   diffs = new uint64_t[amplicons];
-  alignlengths = new uint64_t[amplicons];
+  std::vector<uint64_t> alignlengths(amplicons);
   qgramamps = new uint64_t[amplicons];
   qgramdiffs = new uint64_t[amplicons];
   qgramindices = new uint64_t[amplicons];
-  auto * hits = new uint64_t[amplicons];
+  std::vector<uint64_t> hits(amplicons);
 
   auto diff_saturation
-    = static_cast<uint64_t>(std::min(UINT8_MAX / p.penalty_mismatch,
+    = static_cast<uint64_t>(std::min(UINT8_MAX / parameters.penalty_mismatch,
                                      UINT8_MAX / (penalty_gapopen +
                                                   penalty_gapextend)));
 
@@ -247,11 +279,11 @@ void algo_run(struct Parameters const & p)
   }
 
   /* always search in 8 bit mode unless resolution is very high */
-  constexpr int bit_mode_8 {8};
-  constexpr int bit_mode_16 {16};
+  static constexpr int bit_mode_8 {8};
+  static constexpr int bit_mode_16 {16};
   int bits {bit_mode_8};
 
-  if (static_cast<uint64_t>(p.opt_differences) > diff_saturation) {
+  if (static_cast<uint64_t>(parameters.opt_differences) > diff_saturation) {
     bits = bit_mode_16;
   }
 
@@ -271,7 +303,7 @@ void algo_run(struct Parameters const & p)
 
       /* process each initial seed */
 
-      swarmid++;
+      ++swarmid;
 
       uint64_t swarmsize {0};
       uint64_t amplicons_copies {0};
@@ -282,23 +314,23 @@ void algo_run(struct Parameters const & p)
       uint64_t seedindex {0};
 
       seedindex = seeded;
-      seeded++;
+      ++seeded;
 
       amps[seedindex].swarmid = swarmid;
       amps[seedindex].generation = 0;
       amps[seedindex].radius = 0;
 
-      uint64_t seedampliconid = amps[seedindex].ampliconid;
+      const uint64_t seedampliconid = amps[seedindex].ampliconid;
       hits[hitcount++] = seedampliconid;
 
       uint64_t abundance = db_getabundance(seedampliconid);
       amplicons_copies += abundance;
       if (abundance == 1) {
-        singletons++;
+        ++singletons;
       }
 
       swarmsize = 1;
-      swarmed++;
+      ++swarmed;
 
 
       /* find diff estimates between seed and each amplicon in pool */
@@ -309,40 +341,34 @@ void algo_run(struct Parameters const & p)
 
       for(auto i = 0ULL; i < amplicons - swarmed; i++)
         {
-          unsigned ampid = amps[swarmed+i].ampliconid;
-          if ((opt_no_otu_breaking) || (db_getabundance(ampid) <= abundance))
+          const unsigned int ampid = amps[swarmed + i].ampliconid;
+          if ((opt_no_otu_breaking) or (db_getabundance(ampid) <= abundance))
             {
               qgramamps[i] = ampid;
-              listlen++;
+              ++listlen;
             }
         }
 
       qgram_diff_fast(seedampliconid, listlen, qgramamps, qgramdiffs);
 
-#ifdef VERBOSE
-      estimates += listlen;
-#endif
 
       for(auto i = 0ULL; i < listlen; i++)
         {
-          uint64_t poolampliconid = qgramamps[i];
-          uint64_t diff = qgramdiffs[i];
-          amps[swarmed+i].diffestimate = static_cast<unsigned int>(diff);
-          if (diff <= static_cast<uint64_t>(p.opt_differences))
+          const uint64_t poolampliconid = qgramamps[i];
+          const uint64_t diff = qgramdiffs[i];
+          amps[swarmed + i].diffestimate = static_cast<unsigned int>(diff);
+          if (diff <= static_cast<uint64_t>(parameters.opt_differences))
             {
-              targetindices[targetcount] = swarmed+i;
+              targetindices[targetcount] = swarmed + i;
               targetampliconids[targetcount] = poolampliconid;
-              targetcount++;
+              ++targetcount;
             }
         }
 
       if (targetcount > 0)
         {
-          search_do(seedampliconid, targetcount, targetampliconids,
-                    scores, diffs, alignlengths, bits);
-#ifdef VERBOSE
-          searches++;
-#endif
+          search_do(seedampliconid, targetcount, targetampliconids.data(),
+                    scores, diffs, alignlengths.data(), bits);
 
           if (bits == bit_mode_8) {
             count_comparisons_8 += targetcount;
@@ -353,33 +379,18 @@ void algo_run(struct Parameters const & p)
 
           for(auto t = 0ULL; t < targetcount; t++)
             {
-#if 0
-              printf("seed: %" PRIu64
-                     " target: %" PRIu64
-                     " score: %" PRIu64
-                     " diffs: %" PRIu64
-                     " alignlen: %" PRIu64
-                     " bits: %" PRIu64 "\n",
-                     seedampliconid,
-                     targetampliconids[t],
-                     scores[t],
-                     diffs[t],
-                     alignlengths[t],
-                     bits);
-#endif
+              const uint64_t diff = diffs[t];
 
-              uint64_t diff = diffs[t];
-
-              if (diff <= static_cast<uint64_t>(p.opt_differences))
+              if (diff <= static_cast<uint64_t>(parameters.opt_differences))
                 {
-                  uint64_t i = targetindices[t];
+                  const uint64_t i = targetindices[t];
 
                   /* move the target (i) to the position (swarmed)
                      of the first unswarmed amplicon in the pool */
 
                   if (swarmed < i)
                     {
-                      struct ampliconinfo_s temp = amps[i];
+                      const struct ampliconinfo_s temp = amps[i];
                       for(auto j = i; j > swarmed; j--)
                         {
                           amps[j] = amps[j-1];
@@ -397,32 +408,32 @@ void algo_run(struct Parameters const & p)
                     maxradius = diff;
                   }
 
-                  unsigned poolampliconid = amps[swarmed].ampliconid;
+                  const unsigned int poolampliconid = amps[swarmed].ampliconid;
                   hits[hitcount++] = poolampliconid;
 
-                  if (! p.opt_internal_structure.empty())
+                  if (not parameters.opt_internal_structure.empty())
                     {
                       fprint_id_noabundance(internal_structure_file,
-                                            seedampliconid, p.opt_usearch_abundance);
-                      fprintf(internal_structure_file, "\t");
+                                            seedampliconid, parameters.opt_usearch_abundance);
+                      std::fprintf(internal_structure_file, "\t");
                       fprint_id_noabundance(internal_structure_file,
-                                            poolampliconid, p.opt_usearch_abundance);
-                      fprintf(internal_structure_file, "\t%" PRIu64, diff);
-                      fprintf(internal_structure_file,
+                                            poolampliconid, parameters.opt_usearch_abundance);
+                      std::fprintf(internal_structure_file, "\t%" PRIu64, diff);
+                      std::fprintf(internal_structure_file,
                               "\t%u\t1",
                               swarmid);
-                      fprintf(internal_structure_file, "\n");
+                      std::fprintf(internal_structure_file, "\n");
                     }
 
                   abundance = db_getabundance(poolampliconid);
                   amplicons_copies += abundance;
                   if (abundance == 1) {
-                    singletons++;
+                    ++singletons;
                   }
 
-                  swarmsize++;
+                  ++swarmsize;
 
-                  swarmed++;
+                  ++swarmed;
                 }
             }
 
@@ -445,49 +456,42 @@ void algo_run(struct Parameters const & p)
               subseedgeneration = amps[subseedindex].generation;
               subseedabundance = db_getabundance(subseedampliconid);
 
-              seeded++;
+              ++seeded;
 
               targetcount = 0;
 
               uint64_t subseedlistlen {0};
               for(auto i = swarmed; i < amplicons; i++)
                 {
-                  uint64_t targetampliconid = amps[i].ampliconid;
+                  const uint64_t targetampliconid = amps[i].ampliconid;
                   if ((amps[i].diffestimate <=
-                       subseedradius + p.opt_differences) &&
-                      ((opt_no_otu_breaking) ||
+                       subseedradius + parameters.opt_differences) and
+                      ((opt_no_otu_breaking) or
                        (db_getabundance(targetampliconid)
                         <= subseedabundance)))
                     {
                       qgramamps[subseedlistlen] = targetampliconid;
                       qgramindices[subseedlistlen] = i;
-                      subseedlistlen++;
+                      ++subseedlistlen;
                     }
                 }
 
               qgram_diff_fast(subseedampliconid, subseedlistlen, qgramamps,
                               qgramdiffs);
 
-#ifdef VERBOSE
-              estimates += subseedlistlen;
-#endif
-
               for(auto i = 0ULL; i < subseedlistlen; i++) {
-                if (qgramdiffs[i] <= static_cast<uint64_t>(p.opt_differences))
+                if (qgramdiffs[i] <= static_cast<uint64_t>(parameters.opt_differences))
                   {
                     targetindices[targetcount] = qgramindices[i];
                     targetampliconids[targetcount] = qgramamps[i];
-                    targetcount++;
+                    ++targetcount;
                   }
               }
 
               if (targetcount > 0)
                 {
-                  search_do(subseedampliconid, targetcount, targetampliconids,
-                            scores, diffs, alignlengths, bits);
-#ifdef VERBOSE
-                  searches++;
-#endif
+                  search_do(subseedampliconid, targetcount, targetampliconids.data(),
+                            scores, diffs, alignlengths.data(), bits);
 
                   if (bits == bit_mode_8) {
                     count_comparisons_8 += targetcount;
@@ -498,11 +502,11 @@ void algo_run(struct Parameters const & p)
 
                   for(auto t = 0ULL; t < targetcount; t++)
                     {
-                      uint64_t diff = diffs[t];
+                      const uint64_t diff = diffs[t];
 
-                      if (diff <= static_cast<uint64_t>(p.opt_differences))
+                      if (diff <= static_cast<uint64_t>(parameters.opt_differences))
                         {
-                          uint64_t i = targetindices[t];
+                          const uint64_t i = targetindices[t];
 
                           /* find correct position in list */
 
@@ -512,18 +516,18 @@ void algo_run(struct Parameters const & p)
                              but unseeded part of the list, so that the
                              swarmed amplicons are ordered by id */
 
-                          uint64_t targetampliconid = amps[i].ampliconid;
+                          const uint64_t targetampliconid = amps[i].ampliconid;
                           uint64_t pos = swarmed;
 
-                          while ((pos > seeded) &&
-                                 (amps[pos-1].ampliconid > targetampliconid) &&
+                          while ((pos > seeded) and
+                                 (amps[pos-1].ampliconid > targetampliconid) and
                                  (amps[pos-1].generation > subseedgeneration)) {
-                            pos--;
+                            --pos;
                           }
 
                           if (pos < i)
                             {
-                              struct ampliconinfo_s temp = amps[i];
+                              const struct ampliconinfo_s temp = amps[i];
                               for(auto j = i; j > pos; j--)
                                 {
                                   amps[j] = amps[j-1];
@@ -543,34 +547,34 @@ void algo_run(struct Parameters const & p)
                             maxradius = amps[pos].radius;
                           }
 
-                          unsigned poolampliconid = amps[pos].ampliconid;
+                          const unsigned int poolampliconid = amps[pos].ampliconid;
                           hits[hitcount++] = poolampliconid;
 
-                          if (! p.opt_internal_structure.empty())
+                          if (not parameters.opt_internal_structure.empty())
                             {
                               fprint_id_noabundance(internal_structure_file,
                                                     subseedampliconid,
-                                                    p.opt_usearch_abundance);
-                              fprintf(internal_structure_file, "\t");
+                                                    parameters.opt_usearch_abundance);
+                              std::fprintf(internal_structure_file, "\t");
                               fprint_id_noabundance(internal_structure_file,
                                                     poolampliconid,
-                                                    p.opt_usearch_abundance);
-                              fprintf(internal_structure_file, "\t%" PRIu64, diff);
-                              fprintf(internal_structure_file,
+                                                    parameters.opt_usearch_abundance);
+                              std::fprintf(internal_structure_file, "\t%" PRIu64, diff);
+                              std::fprintf(internal_structure_file,
                                       "\t%u\t%" PRIu64,
                                       swarmid, subseedgeneration + 1);
-                              fprintf(internal_structure_file, "\n");
+                              std::fprintf(internal_structure_file, "\n");
                             }
 
                           abundance = db_getabundance(poolampliconid);
                           amplicons_copies += abundance;
                           if (abundance == 1) {
-                            singletons++;
+                            ++singletons;
                           }
 
-                          swarmsize++;
+                          ++swarmsize;
 
-                          swarmed++;
+                          ++swarmed;
                         }
                     }
                 }
@@ -588,25 +592,25 @@ void algo_run(struct Parameters const & p)
 
       if (uclustfile != nullptr)
         {
-          fprintf(uclustfile, "C\t%u\t%" PRIu64 "\t*\t*\t*\t*\t*\t",
+          std::fprintf(uclustfile, "C\t%u\t%" PRIu64 "\t*\t*\t*\t*\t*\t",
                   swarmid-1, swarmsize);
-          fprint_id(uclustfile, seedampliconid, p.opt_usearch_abundance, p.opt_append_abundance);
-          fprintf(uclustfile, "\t*\n");
+          fprint_id(uclustfile, seedampliconid, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+          std::fprintf(uclustfile, "\t*\n");
 
-          fprintf(uclustfile, "S\t%u\t%u\t*\t*\t*\t*\t*\t",
+          std::fprintf(uclustfile, "S\t%u\t%u\t*\t*\t*\t*\t*\t",
                   swarmid-1, db_getsequencelen(seedampliconid));
-          fprint_id(uclustfile, seedampliconid, p.opt_usearch_abundance, p.opt_append_abundance);
-          fprintf(uclustfile, "\t*\n");
+          fprint_id(uclustfile, seedampliconid, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+          std::fprintf(uclustfile, "\t*\n");
           fflush(uclustfile);
 
           for(auto i = 1ULL; i < hitcount; i++)
             {
-              uint64_t hit = hits[i];
+              const uint64_t hit = hits[i];
 
               char * dseq = db_getsequence(hit);
-              int64_t dlen = db_getsequencelen(hit);
+              const int64_t dlen = db_getsequencelen(hit);
               char * qseq = db_getsequence(seedampliconid);
-              int64_t qlen = db_getsequencelen(seedampliconid);
+              const int64_t qlen = db_getsequencelen(seedampliconid);
 
               int64_t nwscore {0};
               int64_t nwdiff {0};
@@ -618,21 +622,23 @@ void algo_run(struct Parameters const & p)
                  & nwscore, & nwdiff, & nwalignmentlength, & nwalignment,
                  dir, reinterpret_cast<int64_t *>(hearray), 0, 0);
 
-              double percentid
+              const double percentid
                 = 100.0 * static_cast<double>(nwalignmentlength - nwdiff)
                 / static_cast<double>(nwalignmentlength);
 
-              fprintf(uclustfile, "H\t%u\t%u\t%.1f\t+\t0\t0\t%s\t",
+              std::fprintf(uclustfile, "H\t%u\t%u\t%.1f\t+\t0\t0\t%s\t",
                       swarmid-1, db_getsequencelen(hit), percentid,
                       nwdiff > 0 ? nwalignment : "=");
 
-              fprint_id(uclustfile, hit, p.opt_usearch_abundance, p.opt_append_abundance);
-              fprintf(uclustfile, "\t");
-              fprint_id(uclustfile, seedampliconid, p.opt_usearch_abundance, p.opt_append_abundance);
-              fprintf(uclustfile, "\n");
+              fprint_id(uclustfile, hit, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+              std::fprintf(uclustfile, "\t");
+              fprint_id(uclustfile, seedampliconid, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+              std::fprintf(uclustfile, "\n");
               fflush(uclustfile);
 
-              delete [] nwalignment;
+              if (nwalignment != nullptr) {
+                xfree(nwalignment);
+              }
               nwalignment = nullptr;
             }
 
@@ -643,10 +649,10 @@ void algo_run(struct Parameters const & p)
         {
           abundance = db_getabundance(seedampliconid);
 
-          fprintf(statsfile, "%" PRIu64 "\t%" PRIu64 "\t",
+          std::fprintf(statsfile, "%" PRIu64 "\t%" PRIu64 "\t",
                   swarmsize, amplicons_copies);
-          fprint_id_noabundance(statsfile, seedampliconid, p.opt_usearch_abundance);
-          fprintf(statsfile,
+          fprint_id_noabundance(statsfile, seedampliconid, parameters.opt_usearch_abundance);
+          std::fprintf(statsfile,
                   "\t%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\n",
                   abundance, singletons, maxgen, maxradius);
         }
@@ -665,18 +671,18 @@ void algo_run(struct Parameters const & p)
 
   /* output swarms */
   if (amplicons > 0) {
-    if (p.opt_mothur) {
-      write_swarms_mothur_format(amplicons, swarmid, p);
+    if (parameters.opt_mothur) {
+      write_swarms_mothur_format(amplicons, swarmid, parameters);
     }
     else {
-      write_swarms_default_format(amplicons, p);
+      write_swarms_default_format(amplicons, parameters);
     }
   }
 
 
   /* dump seeds in fasta format with sum of abundances */
-  if ((not p.opt_seeds.empty()) && (amplicons > 0)) {
-    write_representative_sequences(amplicons, p);
+  if ((not parameters.opt_seeds.empty()) and (amplicons > 0)) {
+    write_representative_sequences(amplicons, parameters);
   }
 
 
@@ -686,18 +692,10 @@ void algo_run(struct Parameters const & p)
   qgramamps = nullptr;
   delete [] qgramindices;
   qgramindices = nullptr;
-  delete [] hits;
-  hits = nullptr;
-  delete [] alignlengths;
-  alignlengths = nullptr;
   delete [] diffs;
   diffs = nullptr;
   delete [] scores;
   scores = nullptr;
-  delete [] targetindices;
-  targetindices = nullptr;
-  delete [] targetampliconids;
-  targetampliconids = nullptr;
   delete [] amps;
   amps = nullptr;
 
@@ -705,38 +703,14 @@ void algo_run(struct Parameters const & p)
 
   qgram_diff_done();
 
-  fprintf(logfile, "\n");
+  std::fprintf(logfile, "\n");
 
-  fprintf(logfile, "Number of swarms:  %u\n", swarmid);
+  std::fprintf(logfile, "Number of swarms:  %u\n", swarmid);
 
-  fprintf(logfile, "Largest swarm:     %" PRIu64 "\n", largestswarm);
+  std::fprintf(logfile, "Largest swarm:     %" PRIu64 "\n", largestswarm);
 
-  fprintf(logfile, "Max generations:   %" PRIu64 "\n", maxgenerations);
+  std::fprintf(logfile, "Max generations:   %" PRIu64 "\n", maxgenerations);
 
-#ifdef VERBOSE
-  fprintf(logfile, "\n");
-
-  fprintf(logfile, "Estimates:         %" PRIu64 "\n", estimates);
-
-  fprintf(logfile, "Searches:          %" PRIu64 "\n", searches);
-
-  fprintf(logfile, "\n");
-
-  fprintf(logfile, "Comparisons (8b):  %" PRIu64 " (%.2lf%%)\n",
-          count_comparisons_8, (200.0 * count_comparisons_8 /
-                                amplicons / (amplicons+1)));
-
-  fprintf(logfile, "Comparisons (16b): %" PRIu64 " (%.2lf%%)\n",
-          count_comparisons_16, (200.0 * count_comparisons_16 /
-                                 amplicons / (amplicons+1)));
-
-  fprintf(logfile, "Comparisons (tot): %" PRIu64 " (%.2lf%%)\n",
-          count_comparisons_8 + count_comparisons_16,
-          (200.0 * (count_comparisons_8 + count_comparisons_16) /
-           amplicons / (amplicons+1)));
-#endif
-
-    search_end();
-    score_matrix_free();
+  search_end();
 }
 
