@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -25,10 +25,32 @@
 #include "qgram.h"
 #include "util.h"
 #include "utils/fatal.h"
+#include "utils/nt_codec.h"
+#include "utils/qgram_array.h"
 #include "zobrist.h"
+#include <algorithm>  // std::min()
+#include <array>
+#include <cassert>  // assert()
+#include <cinttypes>  // macros PRIu64 and PRId64
+#include <cstdint>  // int64_t, uint64_t
+#include <cstdio>  // fileno, fclose(), size_t
+#include <cstdlib>  // qsort()
+#include <cstring>  // memcpy, memcmp
+#include <string>
+#include <sys/stat.h>  // fstat, S_ISREG, stat
+#include <sys/types.h>  // ssize_t
+#include <vector>
+
+#ifndef PRIu64
+#ifdef _WIN32
+#define PRIu64 "I64u"
+#else
+constexpr char PRIu64[] = "lu";
+#endif
+#endif
 
 
-constexpr unsigned int memchunk {1 << 20};  // 1 megabyte
+constexpr unsigned int memchunk {1U << 20U};  // 1 megabyte
 constexpr unsigned int linealloc {2048};
 constexpr long unsigned int n_chars {INT8_MAX + 1};  // 128 ascii chars
 constexpr unsigned int max_sequence_length {67108861};  // (2^26 - 3)
@@ -37,10 +59,9 @@ constexpr unsigned int max_sequence_length {67108861};  // (2^26 - 3)
 // uint32 pointers, which leads to a segmentation fault
 constexpr unsigned int max_header_length {16777216 - 1};  // 2^24 minus 1
 
-auto make_nt_map () -> std::array<signed char, n_chars> {
-    // set the 128 ascii chars to '-1' except Aa, Cc, Gg, Tt and Uu
-  std::array<signed char, n_chars> ascii_map {{0}};
-    ascii_map.fill(-1);
+auto make_nt_map () -> std::array<uint64_t, n_chars> {
+    // set the 128 ascii chars to zero except Aa, Cc, Gg, Tt and Uu
+  std::array<uint64_t, n_chars> ascii_map {{0}};
     ascii_map['A'] = 1;
     ascii_map['a'] = 1;
     ascii_map['C'] = 2;
@@ -92,7 +113,6 @@ using seqinfo_t = struct seqinfo_s;
 extern seqinfo_t * seqindex;
 
 seqinfo_t * seqindex {nullptr};
-qgramvector_t * qgrams {nullptr};
 
 
 auto db_getnucleotidecount() -> uint64_t
@@ -113,51 +133,51 @@ auto db_getlongestsequence() -> unsigned int
 }
 
 
-auto fprint_id(std::FILE * stream, uint64_t x, bool opt_usearch_abundance,
+auto fprint_id(std::FILE * stream, uint64_t seqno, bool opt_usearch_abundance,
                int64_t opt_append_abundance) -> void
 {
-  const seqinfo_t * sp = seqindex + x;
-  const char * h = sp->header;
-  const int hdrlen = sp->headerlen;
+  const seqinfo_t * seqinfo = seqindex + seqno;
+  const char * hdrstr = seqinfo->header;
+  const int hdrlen = seqinfo->headerlen;
 
-  if ((opt_append_abundance != 0) && (sp->abundance_start == sp->abundance_end)) {
+  if ((opt_append_abundance != 0) and (seqinfo->abundance_start == seqinfo->abundance_end)) {
     if (opt_usearch_abundance) {
-      fprintf(stream, "%.*s;size=%" PRIu64 ";", hdrlen, h, sp->abundance);
+      std::fprintf(stream, "%.*s;size=%" PRIu64 ";", hdrlen, hdrstr, seqinfo->abundance);
     }
     else {
-      fprintf(stream, "%.*s_%" PRIu64, hdrlen, h, sp->abundance);
+      std::fprintf(stream, "%.*s_%" PRIu64, hdrlen, hdrstr, seqinfo->abundance);
     }
   }
   else {
-    fprintf(stream, "%.*s", hdrlen, h);
+    std::fprintf(stream, "%.*s", hdrlen, hdrstr);
   }
 }
 
 
 auto fprint_id_noabundance(std::FILE * stream, uint64_t seqno, bool opt_usearch_abundance) -> void
 {
-  const seqinfo_t * sp = seqindex + seqno;
-  const char * h = sp->header;
-  const int hdrlen = sp->headerlen;
+  const seqinfo_t * seqinfo = seqindex + seqno;
+  const char * hdrstr = seqinfo->header;
+  const int hdrlen = seqinfo->headerlen;
 
-  if (sp->abundance_start < sp->abundance_end)
+  if (seqinfo->abundance_start < seqinfo->abundance_end)
     {
       /* print start of header */
-      fprintf(stream, "%.*s", sp->abundance_start, h);
+      std::fprintf(stream, "%.*s", seqinfo->abundance_start, hdrstr);
 
       if (opt_usearch_abundance)
         {
           /* print semicolon if the abundance is not at either end */
-          if ((sp->abundance_start > 0) && (sp->abundance_end < hdrlen)) {
-            fprintf(stream, ";");
+          if ((seqinfo->abundance_start > 0) and (seqinfo->abundance_end < hdrlen)) {
+            std::fprintf(stream, ";");
           }
 
           /* print remaining part */
-          fprintf(stream, "%.*s", hdrlen - sp->abundance_end, h + sp->abundance_end);
+          std::fprintf(stream, "%.*s", hdrlen - seqinfo->abundance_end, hdrstr + seqinfo->abundance_end);
         }
     }
   else {
-    fprintf(stream, "%.*s", hdrlen, h);
+    std::fprintf(stream, "%.*s", hdrlen, hdrstr);
   }
 }
 
@@ -167,44 +187,42 @@ auto fprint_id_with_new_abundance(std::FILE * stream,
                                   uint64_t abundance,
                                   bool opt_usearch_abundance) -> void
 {
-  const seqinfo_t * sp = seqindex + seqno;
+  const seqinfo_t * seqinfo = seqindex + seqno;
 
   if (opt_usearch_abundance) {
-    fprintf(stream,
+    std::fprintf(stream,
             "%.*s%ssize=%" PRIu64 ";%.*s",
-            sp->abundance_start,
-            sp->header,
-            sp->abundance_start > 0 ? ";" : "",
+            seqinfo->abundance_start,
+            seqinfo->header,
+            seqinfo->abundance_start > 0 ? ";" : "",
             abundance,
-            sp->headerlen - sp->abundance_end,
-            sp->header + sp->abundance_end);
+            seqinfo->headerlen - seqinfo->abundance_end,
+            seqinfo->header + seqinfo->abundance_end);
   }
   else {
-    fprintf(stream,
+    std::fprintf(stream,
             "%.*s_%" PRIu64,
-            sp->abundance_start,
-            sp->header,
+            seqinfo->abundance_start,
+            seqinfo->header,
             abundance);
   }
 }
 
 
-auto db_compare_abundance(const void * a, const void * b) -> int
+auto db_compare_abundance(const void * ptr_a, const void * ptr_b) -> int
 {
-  const auto * x = reinterpret_cast<const seqinfo_t *>(a);
-  const auto * y = reinterpret_cast<const seqinfo_t *>(b);
-  int status {0};
+  const auto * seqinfo_x = reinterpret_cast<const seqinfo_t *>(ptr_a);
+  const auto * seqinfo_y = reinterpret_cast<const seqinfo_t *>(ptr_b);
 
-  if (x->abundance > y->abundance) {
-    status = -1;
+  if (seqinfo_x->abundance > seqinfo_y->abundance) {
+    return -1;
   }
-  else if (x->abundance < y->abundance) {
-    status = +1;
+
+  if (seqinfo_x->abundance < seqinfo_y->abundance) {
+    return +1;
   }
-  else {
-    status = strcmp(x->header, y->header);
-  }
-  return status;
+
+  return std::strcmp(seqinfo_x->header, seqinfo_y->header);
 }
 
 
@@ -222,35 +240,35 @@ auto find_swarm_abundance(const char * header,
   * end = 0;
   * number = 0;
 
-  constexpr unsigned int max_digits {20};  // 20 digits at most (abundance > 10^20)
-  const std::string digit_chars = "0123456789";
+  static constexpr unsigned int max_digits {20};  // 20 digits at most (abundance > 10^20)
+  static const std::string digit_chars = "0123456789";
 
   if (header == nullptr) {
+    return false;  // refactoring: if header cannot be a nullptr, replace with assert
+  }
+
+  const char * abundance_string = std::strrchr(header, '_');
+
+  if (abundance_string == nullptr) {
     return false;
   }
 
-  const char * us = strrchr(header, '_');
+  const std::size_t n_digits = std::strspn(abundance_string + 1, digit_chars.c_str());
 
-  if (us == nullptr) {
+  if (n_digits > max_digits) {
     return false;
   }
 
-  size_t digits = strspn(us + 1, digit_chars.c_str());
-
-  if (digits > max_digits) {
+  if (abundance_string[n_digits + 1] != 0) {
     return false;
   }
 
-  if (us[digits + 1] != 0) {
-    return false;
-  }
+  const int64_t abundance_start = abundance_string - header;
+  const int64_t abundance_end = abundance_start + 1 + static_cast<int64_t>(n_digits);
 
-  int64_t s = us - header;
-  int64_t e = s + 1 + static_cast<int64_t>(digits);
-
-  * start = static_cast<int>(s);
-  * end = static_cast<int>(e);
-  * number = atol(us + 1);
+  * start = static_cast<int>(abundance_start);
+  * end = static_cast<int>(abundance_end);
+  * number = std::atol(abundance_string + 1);
 
   return true;
 }
@@ -268,55 +286,56 @@ auto find_usearch_abundance(const char * header,
 
   assert(header != nullptr); // header cannot be a nullptr at this stage
 
-  const std::string attribute {"size="};
-  const std::string digit_chars {"0123456789"};
-  const uint64_t hlen = strlen(header);
+  static const std::string attribute {"size="};
+  static const std::string digit_chars {"0123456789"};
+  const uint64_t hlen = std::strlen(header);
   const uint64_t alen = attribute.length();
-  uint64_t i = 0;
+  uint64_t position = 0;
 
-  while (i + alen < hlen)
+  while (position + alen < hlen)
     {
-      const char * r = strstr(header + i, attribute.c_str());
+      const char * result = std::strstr(header + position, attribute.c_str());
 
       /* no match */
-      if (r == nullptr) {
+      if (result == nullptr) {
         break;
       }
 
-      i = static_cast<uint64_t>(r - header);
+      position = static_cast<uint64_t>(result - header);
 
       /* check for ';' in front */
-      if ((i > 0) && (header[i-1] != ';'))
+      if ((position > 0) and (header[position - 1] != ';'))
         {
-          i += alen + 1;
+          position += alen + 1;
           continue;
         }
 
-      uint64_t digits = strspn(header + i + alen, digit_chars.c_str());
+      const uint64_t n_digits = std::strspn(header + position + alen, digit_chars.c_str());
 
       /* check for at least one digit */
-      if (digits == 0)
+      if (n_digits == 0)
         {
-          i += alen + 1;
+          position += alen + 1;
           continue;
         }
 
       /* check for ';' after */
-      if ((i + alen + digits < hlen) && (header[i + alen + digits] != ';'))
+      if ((position + alen + n_digits < hlen) and (header[position + alen + n_digits] != ';'))
         {
-          i += alen + digits + 2;
+          position += alen + n_digits + 2;
           continue;
         }
 
       /* ok */
-      if (i > 0) {
-        * start = static_cast<int>(i - 1);
+      if (position > 0) {
+        * start = static_cast<int>(position - 1);
       }
       else {
         * start = 0;
       }
-      * end   = static_cast<int>(std::min(i + alen + digits + 1, hlen));
-      * number = atol(header + i + alen);
+      * end   = static_cast<int>(std::min(position + alen + n_digits + 1, hlen));
+      * number = std::atol(header + position + alen);
+
       return true;
     }
 
@@ -324,10 +343,10 @@ auto find_usearch_abundance(const char * header,
 }
 
 
-void find_abundance(struct seqinfo_s * sp, uint64_t lineno,
+void find_abundance(struct seqinfo_s * seqinfo, uint64_t lineno,
                     bool opt_usearch_abundance, int64_t opt_append_abundance)
 {
-  char * header = sp->header;
+  char * header = seqinfo->header;
 
   /* read size/abundance annotation */
   int64_t abundance = 0;
@@ -364,7 +383,7 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno,
 
   if (abundance == 0)
     {
-      start = sp->headerlen;
+      start = seqinfo->headerlen;
       end = start;
 
       if (opt_append_abundance != 0) {
@@ -372,7 +391,7 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno,
       }
       else
         {
-          missingabundance++;
+          ++missingabundance;
           // record the position of the first missing abundance entry
           if (missingabundance == 1)
             {
@@ -382,18 +401,18 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno,
         }
     }
 
-  sp->abundance = static_cast<uint64_t>(abundance);
-  sp->abundance_start = start;
-  sp->abundance_end = end;
+  seqinfo->abundance = static_cast<uint64_t>(abundance);
+  seqinfo->abundance_start = start;
+  seqinfo->abundance_end = end;
 }
 
 
-void db_read(const char * filename, struct Parameters const & p)
+auto db_read(const char * filename, struct Parameters const & parameters) -> void
 {
   /* allocate space */
 
   uint64_t dataalloc {memchunk};
-  datap = new char[dataalloc];
+  datap = static_cast<char *>(xmalloc(dataalloc));
   uint64_t datalen {0};
   uint64_t duplicates_found {0};
 
@@ -415,21 +434,21 @@ void db_read(const char * filename, struct Parameters const & p)
 
   /* get file size */
 
-  struct stat fs;
+  struct stat fstat_buffer;  // refactoring: add initializer '{}' (warning with GCC < 5)
 
-  if (fstat(fileno(input_fp), & fs) != 0)
+  if (fstat(fileno(input_fp), &fstat_buffer) != 0)  // refactor: fstat and fileno linuxisms
     {
       fatal(error_prefix, "Unable to fstat on input file (", filename, ").\n");
     }
-  bool is_regular = S_ISREG(fs.st_mode);
-  uint64_t filesize = is_regular ? fs.st_size : 0;
+  const bool is_regular = S_ISREG(fstat_buffer.st_mode);  // refactor: S_ISREG linuxisms
+  const uint64_t filesize = is_regular ? fstat_buffer.st_size : 0;
   uint64_t filepos = 0;
 
-  if (! is_regular) {
-    fprintf(logfile, "Waiting for data... (hit Ctrl-C and run 'swarm -h' if you meant to read data from a file)\n");
+  if (not is_regular) {
+    std::fprintf(logfile, "Waiting for data... (hit Ctrl-C and run 'swarm -h' if you meant to read data from a file)\n");
   }
 
-  size_t linecap = linealloc;
+  std::size_t linecap = linealloc;
   char * line {new char[linecap]};
   ssize_t linelen = xgetline(& line, & linecap, input_fp);
   if (linelen < 0)
@@ -453,7 +472,7 @@ void db_read(const char * filename, struct Parameters const & p)
       }
 
       auto headerlen = static_cast<unsigned int>
-        (strcspn(line + 1, " \r\n"));
+        (std::strcspn(line + 1, " \r\n"));
 
       headerchars += headerlen;
 
@@ -472,7 +491,7 @@ void db_read(const char * filename, struct Parameters const & p)
           dataalloc += memchunk;
           datap = static_cast<char *>(xrealloc(datap, dataalloc));
         }
-      memcpy(datap + datalen, & lineno, sizeof(unsigned int));
+      std::memcpy(datap + datalen, & lineno, sizeof(unsigned int));
       datalen += sizeof(unsigned int);
 
 
@@ -483,7 +502,7 @@ void db_read(const char * filename, struct Parameters const & p)
           dataalloc += memchunk;
           datap = static_cast<char *>(xrealloc(datap, dataalloc));
         }
-      memcpy(datap + datalen, line + 1, headerlen);
+      std::memcpy(datap + datalen, line + 1, headerlen);
       *(datap + datalen + headerlen) = 0;
       datalen += headerlen + 1;
 
@@ -498,7 +517,7 @@ void db_read(const char * filename, struct Parameters const & p)
         }
       filepos += linelen;
 
-      lineno++;
+      ++lineno;
 
 
       /* store a dummy sequence length */
@@ -510,8 +529,8 @@ void db_read(const char * filename, struct Parameters const & p)
           dataalloc += memchunk;
           datap = static_cast<char *>(xrealloc(datap, dataalloc));
         }
-      uint64_t datalen_seqlen = datalen;
-      memcpy(datap + datalen, & length, sizeof(unsigned int));
+      const uint64_t datalen_seqlen = datalen;
+      std::memcpy(datap + datalen, & length, sizeof(unsigned int));
       datalen += sizeof(unsigned int);
 
 
@@ -520,23 +539,23 @@ void db_read(const char * filename, struct Parameters const & p)
       uint64_t nt_buffer {0};
       unsigned int nt_bufferlen {0};
       const unsigned int nt_buffersize {4 * sizeof(nt_buffer)};
-      constexpr int new_line {10};
-      constexpr int carriage_return {13};
-      constexpr int start_chars_range {32};  // visible ascii chars: 32-126
-      constexpr int end_chars_range {126};
+      static constexpr int new_line {10};
+      static constexpr int carriage_return {13};
+      static constexpr int start_chars_range {32};  // visible ascii chars: 32-126
+      static constexpr int end_chars_range {126};
 
-      while ((line[0] != 0) && (line[0] != '>'))
+      while ((line[0] != 0) and (line[0] != '>'))
         {
-          unsigned char c {0};
-          char * pl = line;
-          while((c = static_cast<unsigned char>(*pl++)) != 0U)
+          unsigned char character {0};
+          char * line_ptr = line;
+          while((character = static_cast<signed char>(*line_ptr++)) != 0)
             {
-              signed char m {0};
-              if ((m = map_nt[static_cast<unsigned int>(c)]) >= 0)
+              const auto mapped_char = map_nt[character];
+              if (mapped_char != 0)
                 {
-                  nt_buffer |= ((static_cast<uint64_t>(m))-1) << (2 * nt_bufferlen);
-                  length++;
-                  nt_bufferlen++;
+                  nt_buffer |= (mapped_char - 1) << (2 * nt_bufferlen);
+                  ++length;
+                  ++nt_bufferlen;
 
                   if (nt_bufferlen == nt_buffersize)
                     {
@@ -546,21 +565,21 @@ void db_read(const char * filename, struct Parameters const & p)
                           datap = static_cast<char *>(xrealloc(datap, dataalloc));
                         }
 
-                      memcpy(datap + datalen, & nt_buffer, sizeof(nt_buffer));
+                      std::memcpy(datap + datalen, & nt_buffer, sizeof(nt_buffer));
                       datalen += sizeof(nt_buffer);
 
                       nt_bufferlen = 0;
                       nt_buffer = 0;
                     }
                 }
-              else if ((c != new_line) && (c != carriage_return))
+              else if ((character != new_line) and (character != carriage_return))
                 {
-                  if ((c >= start_chars_range) && (c <= end_chars_range)) {
-                    fatal(error_prefix, "Illegal character '", static_cast<char>(c),
+                  if ((character >= start_chars_range) and (character <= end_chars_range)) {
+                    fatal(error_prefix, "Illegal character '", character,
                           "' in sequence on line ", lineno, ".");
                   }
                   else {
-                    fatal(error_prefix, "Illegal character (ascii no ", static_cast<char>(c),
+                    fatal(error_prefix, "Illegal character (ascii no ", character,
                           ") in sequence on line ", lineno, ".");
                   }
                 }
@@ -579,12 +598,12 @@ void db_read(const char * filename, struct Parameters const & p)
             }
           filepos += linelen;
 
-          lineno++;
+          ++lineno;
         }
 
       /* fill in real length */
 
-      memcpy(datap + datalen_seqlen, & length, sizeof(unsigned int));
+      std::memcpy(datap + datalen_seqlen, & length, sizeof(unsigned int));
 
       if (length == 0)
         {
@@ -608,14 +627,14 @@ void db_read(const char * filename, struct Parameters const & p)
               datap = static_cast<char *>(xrealloc(datap, dataalloc));
             }
 
-          memcpy(datap + datalen, & nt_buffer, sizeof(nt_buffer));
+          std::memcpy(datap + datalen, & nt_buffer, sizeof(nt_buffer));
           datalen += sizeof(nt_buffer);
 
           nt_buffer = 0;
           nt_bufferlen = 0;  // that value is never read again, all tests pass without it
         }
 
-      sequences++;
+      ++sequences;
 
       if (is_regular) {
         progress_update(filepos);
@@ -623,27 +642,27 @@ void db_read(const char * filename, struct Parameters const & p)
     }
   progress_done();
 
-  fclose(input_fp);
+  std::fclose(input_fp);
 
   /* init zobrist hashing */
 
-  // add 2 for two insertions
-  unsigned int zobrist_len = std::max(4 * longestheader, longest + 2);
+  // add 2 for two insertions (refactoring: insertions in headers?)
+  const unsigned int zobrist_len = std::max(4 * longestheader, longest + 2);
   zobrist_init(zobrist_len);
 
   /* set up hash to check for unique headers */
 
-  uint64_t hdrhashsize {2 * sequences};
+  const uint64_t hdrhashsize {2 * sequences};
 
   auto * * hdrhashtable = new seqinfo_t*[hdrhashsize] { };
 
   /* set up hash to check for unique sequences */
 
-  uint64_t seqhashsize {2 * sequences};
+  const uint64_t seqhashsize {2 * sequences};
 
   seqinfo_t * * seqhashtable {nullptr};
 
-  if (p.opt_differences > 1)
+  if (parameters.opt_differences > 1)
     {
       seqhashtable = new seqinfo_t*[seqhashsize] { };
     }
@@ -657,44 +676,44 @@ void db_read(const char * filename, struct Parameters const & p)
 
   bool presorted {true};
 
-  char * pl {datap};
+  char * pl {datap};  // refactoring: purpose and how to rename?
   progress_init("Indexing database:", sequences);
   for(auto i = 0ULL; i < sequences; i++)
     {
       /* get line number */
-      unsigned int line_number = *(reinterpret_cast<unsigned int*>(pl));
+      const unsigned int line_number = *(reinterpret_cast<unsigned int*>(pl));
       pl += sizeof(unsigned int);
 
       /* get header */
       seqindex_p->header = pl;
-      seqindex_p->headerlen = static_cast<int>(strlen(seqindex_p->header));
+      seqindex_p->headerlen = static_cast<int>(std::strlen(seqindex_p->header));
       pl += seqindex_p->headerlen + 1;
 
       /* and sequence */
-      unsigned int seqlen = *(reinterpret_cast<unsigned int*>(pl));
+      const unsigned int seqlen = *(reinterpret_cast<unsigned int*>(pl));
       seqindex_p->seqlen = seqlen;
       pl += sizeof(unsigned int);
       seqindex_p->seq = pl;
       pl += nt_bytelength(seqlen);
 
       /* get amplicon abundance */
-      find_abundance(seqindex_p, line_number, p.opt_usearch_abundance, p.opt_append_abundance);
+      find_abundance(seqindex_p, line_number, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
 
-      if ((seqindex_p->abundance_start == 0) &&
+      if ((seqindex_p->abundance_start == 0) and
           (seqindex_p->abundance_end == seqindex_p->headerlen)) {
         fatal(error_prefix, "Empty sequence identifier.");
       }
 
       /* check if the sequences are presorted by abundance and header */
 
-      if (presorted && (lastseq != nullptr))
+      if (presorted and (lastseq != nullptr))
         {
           if (lastseq->abundance < seqindex_p->abundance) {
             presorted = false;
           }
           else if (lastseq->abundance == seqindex_p->abundance)
             {
-              if (strcmp(lastseq->header, seqindex_p->header) > 0) {
+              if (std::strcmp(lastseq->header, seqindex_p->header) > 0) {
                 presorted = false;
               }
             }
@@ -722,9 +741,9 @@ void db_read(const char * filename, struct Parameters const & p)
           id_len = seqindex_p->headerlen - seqindex_p->abundance_end;
         }
 
-      uint64_t hdrhash = zobrist_hash(reinterpret_cast<unsigned char*>
-                                      (seqindex_p->header + id_start),
-                                      4 * static_cast<unsigned int>(id_len));
+      const uint64_t hdrhash = zobrist_hash(reinterpret_cast<unsigned char*>
+                                            (seqindex_p->header + id_start),
+                                            4 * static_cast<unsigned int>(id_len));
 
       seqindex_p->hdrhash = hdrhash;
       uint64_t hdrhashindex = hdrhash % hdrhashsize;
@@ -749,10 +768,10 @@ void db_read(const char * filename, struct Parameters const & p)
                   hit_id_len = hdrfound->headerlen - hdrfound->abundance_end;
                 }
 
-              if ((id_len == hit_id_len) &&
-                  (strncmp(seqindex_p->header + id_start,
-                           hdrfound->header + hit_id_start,
-                           static_cast<uint64_t>(id_len)) == 0)) {
+              if ((id_len == hit_id_len) and
+                  (std::strncmp(seqindex_p->header + id_start,
+                                hdrfound->header + hit_id_start,
+                                static_cast<uint64_t>(id_len)) == 0)) {
                 break;
               }
             }
@@ -773,7 +792,7 @@ void db_read(const char * filename, struct Parameters const & p)
                                          (seqindex_p->seq),
                                          seqindex_p->seqlen);
 
-      if (p.opt_differences > 1)
+      if (parameters.opt_differences > 1)
         {
           /* Check for duplicated sequences using hash table, */
           /* but only for d>1. Handled internally for d=1.    */
@@ -783,9 +802,9 @@ void db_read(const char * filename, struct Parameters const & p)
 
           while ((seqfound = seqhashtable[seqhashindex]) != nullptr)
             {
-              if ((seqfound->seqhash == seqindex_p->seqhash) &&
-                  (seqfound->seqlen == seqindex_p->seqlen) &&
-                  (memcmp(seqfound->seq,
+              if ((seqfound->seqhash == seqindex_p->seqhash) and
+                  (seqfound->seqlen == seqindex_p->seqlen) and
+                  (std::memcmp(seqfound->seq,
                           seqindex_p->seq,
                           nt_bytelength(seqindex_p->seqlen)) == 0)) {
                 break;
@@ -795,13 +814,13 @@ void db_read(const char * filename, struct Parameters const & p)
 
           if (seqfound != nullptr)
             {
-              duplicates_found++;
+              ++duplicates_found;
               break;
             }
           seqhashtable[seqhashindex] = seqindex_p;
         }
 
-      seqindex_p++;
+      ++seqindex_p;
       progress_update(i);
     }
 
@@ -835,10 +854,10 @@ void db_read(const char * filename, struct Parameters const & p)
             "and the first space or the end of the line, whichever comes first.");
     }
 
-  if (!presorted)
+  if (not presorted)
     {
       progress_init("Abundance sorting:", 1);
-      qsort(seqindex, sequences, sizeof(seqinfo_t), db_compare_abundance);
+      std::qsort(seqindex, sequences, sizeof(seqinfo_t), db_compare_abundance);
       progress_done();
     }
 
@@ -849,9 +868,9 @@ void db_read(const char * filename, struct Parameters const & p)
   seqhashtable = nullptr;
 
   // user report
-  fprintf(logfile, "Database info:     %" PRIu64 " nt", db_getnucleotidecount());
-  fprintf(logfile, " in %u sequences,", db_getsequencecount());
-  fprintf(logfile, " longest %u nt\n", db_getlongestsequence());
+  std::fprintf(logfile, "Database info:     %" PRIu64 " nt", db_getnucleotidecount());
+  std::fprintf(logfile, " in %u sequences,", db_getsequencecount());
+  std::fprintf(logfile, " longest %u nt\n", db_getlongestsequence());
 }
 
 
@@ -867,7 +886,7 @@ void db_qgrams_init()
       findqgrams(reinterpret_cast<unsigned char*>(seqindex_p->seq),
                  seqindex_p->seqlen,
                  qgrams[i]);
-      seqindex_p++;
+      ++seqindex_p;
       progress_update(i);
     }
   progress_done();
@@ -924,7 +943,9 @@ void db_free()
 {
   zobrist_exit();
 
-  delete [] datap;
+  if (datap != nullptr) {
+    xfree(datap);
+  }
   datap = nullptr;
   delete [] seqindex;
   seqindex = nullptr;
@@ -943,5 +964,5 @@ auto db_fprintseq(std::FILE * fastaout_fp, const unsigned int seqno) -> void
   }
   buffer[len] = '\0';
 
-  fprintf(fastaout_fp, "%.*s\n", len, buffer.data());
+  std::fprintf(fastaout_fp, "%.*s\n", len, buffer.data());
 }
