@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,12 +21,33 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include <cstdint>
 #include "swarm.h"
 #include "db.h"
 #include "util.h"
 #include "utils/hashtable_size.h"
+#include "utils/nt_codec.h"
 #include "zobrist.h"
+#include <cinttypes>  // macros PRIu64 and PRId64
+#include <cstdint>
+#include <cstdio>  // fputc()
+#include <cstdlib>  // qsort()
+#include <cstring>  // memcmp
+
+#ifndef PRIu64
+#ifdef _WIN32
+#define PRIu64 "I64u"
+#else
+constexpr char PRIu64[] = "lu";
+#endif
+#endif
+
+#ifndef PRId64
+#ifdef _WIN32
+#define PRId64 "I64d"
+#else
+constexpr char PRId64[] = "ld";
+#endif
+#endif
 
 
 struct bucket
@@ -71,15 +92,15 @@ auto derep_compare(const void * a, const void * b) -> int
 
 
 auto write_stats_file(const uint64_t swarmcount,
-                      struct Parameters const & p,
+                      struct Parameters const & parameters,
                       struct bucket * hashtable) -> void {
   progress_init("Writing stats:    ", swarmcount);
   for(auto i = 0ULL; i < swarmcount; i++)
     {
       struct bucket * sp = hashtable + i;
-      fprintf(statsfile, "%u\t%" PRIu64 "\t", sp->size, sp->mass);
-      fprint_id_noabundance(statsfile, sp->seqno_first, p.opt_usearch_abundance);
-      fprintf(statsfile, "\t%" PRIu64 "\t%u\t%u\t%u\n",
+      std::fprintf(statsfile, "%u\t%" PRIu64 "\t", sp->size, sp->mass);
+      fprint_id_noabundance(statsfile, sp->seqno_first, parameters.opt_usearch_abundance);
+      std::fprintf(statsfile, "\t%" PRIu64 "\t%u\t%u\t%u\n",
               db_getabundance(sp->seqno_first),
               sp->singletons, 0U, 0U);
       progress_update(i);
@@ -89,7 +110,7 @@ auto write_stats_file(const uint64_t swarmcount,
 
 
 auto write_structure_file(const uint64_t swarmcount,
-                          struct Parameters const & p,
+                          struct Parameters const & parameters,
                           struct bucket * hashtable,
                           unsigned int * nextseqtab) -> void {
   progress_init("Writing structure:", swarmcount);
@@ -97,14 +118,14 @@ auto write_structure_file(const uint64_t swarmcount,
   for(uint64_t i = 0; i < swarmcount; i++)
     {
       struct bucket * sp = hashtable + i;
-      uint64_t seed = sp->seqno_first;
+      const uint64_t seed = sp->seqno_first;
       unsigned int a = nextseqtab[seed];
       while (a != 0U)
         {
-          fprint_id_noabundance(internal_structure_file, seed, p.opt_usearch_abundance);
-          fprintf(internal_structure_file, "\t");
-          fprint_id_noabundance(internal_structure_file, a, p.opt_usearch_abundance);
-          fprintf(internal_structure_file, "\t%d\t%" PRIu64 "\t%d\n", 0, i+1, 0);
+          fprint_id_noabundance(internal_structure_file, seed, parameters.opt_usearch_abundance);
+          std::fprintf(internal_structure_file, "\t");
+          fprint_id_noabundance(internal_structure_file, a, parameters.opt_usearch_abundance);
+          std::fprintf(internal_structure_file, "\t%d\t%" PRIu64 "\t%d\n", 0, i + 1, 0);
           a = nextseqtab[a];
         }
       progress_update(i);
@@ -114,7 +135,7 @@ auto write_structure_file(const uint64_t swarmcount,
 
 
 auto write_swarms_uclust_format(const uint64_t swarmcount,
-                                struct Parameters const & p,
+                                struct Parameters const & parameters,
                                 struct bucket * hashtable,
                                 unsigned int * nextseqtab) -> void {
   progress_init("Writing UCLUST:   ", swarmcount);
@@ -123,115 +144,115 @@ auto write_swarms_uclust_format(const uint64_t swarmcount,
     {
       struct bucket * bp = hashtable + swarmid;
 
-      unsigned int seed = bp->seqno_first;
+      const unsigned int seed = bp->seqno_first;
 
-      fprintf(uclustfile, "C\t%u\t%u\t*\t*\t*\t*\t*\t",
+      std::fprintf(uclustfile, "C\t%u\t%u\t*\t*\t*\t*\t*\t",
               swarmid,
               bp->size);
-      fprint_id(uclustfile, seed, p.opt_usearch_abundance, p.opt_append_abundance);
-      fprintf(uclustfile, "\t*\n");
+      fprint_id(uclustfile, seed, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+      std::fprintf(uclustfile, "\t*\n");
 
-      fprintf(uclustfile, "S\t%u\t%u\t*\t*\t*\t*\t*\t",
+      std::fprintf(uclustfile, "S\t%u\t%u\t*\t*\t*\t*\t*\t",
               swarmid,
               db_getsequencelen(seed));
-      fprint_id(uclustfile, seed, p.opt_usearch_abundance, p.opt_append_abundance);
-      fprintf(uclustfile, "\t*\n");
+      fprint_id(uclustfile, seed, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+      std::fprintf(uclustfile, "\t*\n");
 
       unsigned int a = nextseqtab[seed];
 
       while (a != 0U)
         {
-          fprintf(uclustfile,
+          std::fprintf(uclustfile,
                   "H\t%u\t%u\t%.1f\t+\t0\t0\t%s\t",
                   swarmid,
                   db_getsequencelen(a),
                   100.0,
                   "=");
-          fprint_id(uclustfile, a, p.opt_usearch_abundance, p.opt_append_abundance);
-          fprintf(uclustfile, "\t");
-          fprint_id(uclustfile, seed, p.opt_usearch_abundance, p.opt_append_abundance);
-          fprintf(uclustfile, "\n");
+          fprint_id(uclustfile, a, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+          std::fprintf(uclustfile, "\t");
+          fprint_id(uclustfile, seed, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
+          std::fprintf(uclustfile, "\n");
           a = nextseqtab[a];
         }
 
-      progress_update(swarmid+1);
+      progress_update(swarmid + 1);
     }
   progress_done();
 }
 
 
 auto write_representative_sequences(const uint64_t swarmcount,
-                                    struct Parameters const & p,
+                                    struct Parameters const & parameters,
                                     struct bucket * hashtable) -> void {
   progress_init("Writing seeds:    ", swarmcount);
   for(auto i = 0U; i < swarmcount; i++)
     {
-      unsigned int seed = hashtable[i].seqno_first;
-      fprintf(fp_seeds, ">");
-      fprint_id_with_new_abundance(fp_seeds, seed, hashtable[i].mass, p.opt_usearch_abundance);
-      fprintf(fp_seeds, "\n");
+      const unsigned int seed = hashtable[i].seqno_first;
+      std::fprintf(fp_seeds, ">");
+      fprint_id_with_new_abundance(fp_seeds, seed, hashtable[i].mass, parameters.opt_usearch_abundance);
+      std::fprintf(fp_seeds, "\n");
       db_fprintseq(fp_seeds, seed);
-      progress_update(i+1);
+      progress_update(i + 1);
     }
   progress_done();
 }
 
 
 auto write_swarms_mothur_format(const uint64_t swarmcount,
-                                struct Parameters const & p,
+                                struct Parameters const & parameters,
                                 struct bucket * hashtable,
                                 unsigned int * nextseqtab) -> void {
   progress_init("Writing swarms:   ", swarmcount);
-  fprintf(outfile, "swarm_%" PRId64 "\t%" PRIu64, p.opt_differences, swarmcount);
+  std::fprintf(outfile, "swarm_%" PRId64 "\t%" PRIu64, parameters.opt_differences, swarmcount);
 
   for(auto i = 0U; i < swarmcount; i++)
     {
-      unsigned int seed = hashtable[i].seqno_first;
-      fputc('\t', outfile);
-      fprint_id(outfile, seed, p.opt_usearch_abundance, p.opt_append_abundance);
+      const unsigned int seed = hashtable[i].seqno_first;
+      std::fputc('\t', outfile);
+      fprint_id(outfile, seed, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
       unsigned int a = nextseqtab[seed];
 
       while (a != 0U)
         {
-          fputc(',', outfile);
-          fprint_id(outfile, a, p.opt_usearch_abundance, p.opt_append_abundance);
+          std::fputc(',', outfile);
+          fprint_id(outfile, a, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
           a = nextseqtab[a];
         }
 
-      progress_update(i+1);
+      progress_update(i + 1);
     }
-    fputc('\n', outfile);
+    std::fputc('\n', outfile);
 
   progress_done();
 }
 
 
 auto write_swarms_default_format(const uint64_t swarmcount,
-                                 struct Parameters const & p,
+                                 struct Parameters const & parameters,
                                  struct bucket * hashtable,
                                  unsigned int * nextseqtab) -> void {
   progress_init("Writing swarms:   ", swarmcount);
   for(auto i = 0U; i < swarmcount; i++)
     {
-      unsigned int seed = hashtable[i].seqno_first;
-      fprint_id(outfile, seed, p.opt_usearch_abundance, p.opt_append_abundance);
+      const unsigned int seed = hashtable[i].seqno_first;
+      fprint_id(outfile, seed, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
       unsigned int a = nextseqtab[seed];
 
       while (a != 0U)
         {
-          fputc(sepchar, outfile);
-          fprint_id(outfile, a, p.opt_usearch_abundance, p.opt_append_abundance);
+          std::fputc(sepchar, outfile);
+          fprint_id(outfile, a, parameters.opt_usearch_abundance, parameters.opt_append_abundance);
           a = nextseqtab[a];
         }
-      fputc('\n', outfile);
-      progress_update(i+1);
+      std::fputc('\n', outfile);
+      progress_update(i + 1);
     }
 
   progress_done();
 }
 
 
-void dereplicate(struct Parameters const & p)
+auto dereplicate(struct Parameters const & parameters) -> void
 {
   const uint64_t dbsequencecount = db_getsequencecount();
   const uint64_t hashtablesize {compute_hashtable_size(dbsequencecount)};
@@ -250,7 +271,7 @@ void dereplicate(struct Parameters const & p)
 
   for(auto i = 0U; i < dbsequencecount; i++)
     {
-      unsigned int seqlen = db_getsequencelen(i);
+      const unsigned int seqlen = db_getsequencelen(i);
       char * seq = db_getsequence(i);
 
       /*
@@ -261,21 +282,21 @@ void dereplicate(struct Parameters const & p)
         collision when the number of sequences is about 5e9.
       */
 
-      uint64_t hash = zobrist_hash(reinterpret_cast<unsigned char *>(seq),
-                                   seqlen);
+      const uint64_t hash = zobrist_hash(reinterpret_cast<unsigned char *>(seq),
+                                         seqlen);
 
       uint64_t j = hash & derep_hash_mask;
       struct bucket * bp = hashtable + j;
 
-      while (((bp->mass) != 0U) &&
-             ((bp->hash != hash) ||
-              (seqlen != db_getsequencelen(bp->seqno_first)) ||
-              (memcmp(seq,
+      while (((bp->mass) != 0U) and
+             ((bp->hash != hash) or
+              (seqlen != db_getsequencelen(bp->seqno_first)) or
+              (std::memcmp(seq,
                       db_getsequence(bp->seqno_first),
                       nt_bytelength(seqlen)) != 0)))
         {
-          bp++;
-          j++;
+          ++bp;
+          ++j;
           if (bp >= hashtable + hashtablesize) // wrap around the table if we reach the end
             {
               bp = hashtable;
@@ -283,7 +304,7 @@ void dereplicate(struct Parameters const & p)
             }
         }
 
-      uint64_t ab = db_getabundance(i);
+      const uint64_t abundance = db_getabundance(i);
 
       if ((bp->mass) != 0U)
         {
@@ -293,19 +314,19 @@ void dereplicate(struct Parameters const & p)
       else
         {
           /* no identical sequences yet, start a new cluster */
-          swarmcount++;
+          ++swarmcount;
           bp->hash = hash;
           bp->seqno_first = i;
           bp->size = 0;
           bp->singletons = 0;
         }
 
-      bp->size++;
+      ++bp->size;
       bp->seqno_last = i;
-      bp->mass += ab;
+      bp->mass += abundance;
 
-      if (ab == 1) {
-        bp->singletons++;
+      if (abundance == 1) {
+        ++bp->singletons;
       }
 
       if (bp->mass > maxmass) {
@@ -321,42 +342,42 @@ void dereplicate(struct Parameters const & p)
   progress_done();
 
   progress_init("Sorting:          ", 1);
-  qsort(hashtable, hashtablesize, sizeof(bucket), derep_compare);
+  std::qsort(hashtable, hashtablesize, sizeof(bucket), derep_compare);
   progress_done();
 
 
   /* dump swarms */
-  if (p.opt_mothur) {
-    write_swarms_mothur_format(swarmcount, p, hashtable, nextseqtab);
+  if (parameters.opt_mothur) {
+    write_swarms_mothur_format(swarmcount, parameters, hashtable, nextseqtab);
   }
   else {
-    write_swarms_default_format(swarmcount, p, hashtable, nextseqtab);
+    write_swarms_default_format(swarmcount, parameters, hashtable, nextseqtab);
   }
 
   /* dump seeds in fasta format with sum of abundances */
-  if (not p.opt_seeds.empty()) {
-    write_representative_sequences(swarmcount, p, hashtable);
+  if (not parameters.opt_seeds.empty()) {
+    write_representative_sequences(swarmcount, parameters, hashtable);
   }
 
   /* output swarm in uclust format */
   if (uclustfile != nullptr) {
-    write_swarms_uclust_format(swarmcount, p, hashtable, nextseqtab);
+    write_swarms_uclust_format(swarmcount, parameters, hashtable, nextseqtab);
   }
 
   /* output internal structure to file */
-  if (not p.opt_internal_structure.empty()) {
-    write_structure_file(swarmcount, p, hashtable, nextseqtab);
+  if (not parameters.opt_internal_structure.empty()) {
+    write_structure_file(swarmcount, parameters, hashtable, nextseqtab);
   }
 
   /* output statistics to file */
   if (statsfile != nullptr) {
-    write_stats_file(swarmcount, p, hashtable);
+    write_stats_file(swarmcount, parameters, hashtable);
   }
 
-  fprintf(logfile, "\n");
-  fprintf(logfile, "Number of swarms:  %" PRIu64 "\n", swarmcount);
-  fprintf(logfile, "Largest swarm:     %u\n", maxsize);
-  fprintf(logfile, "Heaviest swarm:    %" PRIu64 "\n", maxmass);
+  std::fprintf(logfile, "\n");
+  std::fprintf(logfile, "Number of swarms:  %" PRIu64 "\n", swarmcount);
+  std::fprintf(logfile, "Largest swarm:     %u\n", maxsize);
+  std::fprintf(logfile, "Heaviest swarm:    %" PRIu64 "\n", maxmass);
 
   delete [] nextseqtab;
   nextseqtab = nullptr;
