@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,13 +21,18 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include <cstdint>
+#include <array>
+#include <cstdint>  // int64_t
+
+
+// refactor: 'n_cells' is already defined in 'score_matrix.h'
+constexpr auto n_cells_ = 32ULL;  // number of chars in sym_nt
 
 void nw(char * dseq,
         int64_t dlen,
         char * qseq,
         int64_t qlen,
-        int64_t * score_matrix,
+        const std::array<int64_t, n_cells_ * n_cells_> & score_matrix,
         int64_t gapopen,
         int64_t gapextend,
         int64_t * nwscore,
