@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,11 +21,15 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include "swarm.h"
 
 #ifdef __x86_64__
 #ifdef __POPCNT__
 
+#include <popcntintrin.h>
+#include "utils/qgram_array.h"
+#include <cstdint>  // uint64_t
+
+
 /*
   POPCNT specific code for x86-64
 
@@ -34,20 +38,20 @@
 
   This code requires the _mm_popcnt_u64 intrinsic implemented
   with the POPCNT instruction on the CPU. That instruction was
-  available starting with the Nahalem archtecture in 2008.
+  available starting with the Nehalem architecture in 2008.
 */
 
-auto compareqgramvectors_popcnt(unsigned char * a, unsigned char * b) -> uint64_t
+auto compareqgramvectors_popcnt(unsigned char * qgram_a, unsigned char * qgram_b) -> uint64_t
 {
   /* Count number of different bits */
   /* Uses the POPCNT instruction, requires CPU with this feature */
 
-  auto *ap = reinterpret_cast<uint64_t*>(a);
-  auto *bp = reinterpret_cast<uint64_t*>(b);
+  auto *qgram_vector_a_ptr = reinterpret_cast<uint64_t*>(qgram_a);
+  auto *qgram_vector_b_ptr = reinterpret_cast<uint64_t*>(qgram_b);
   uint64_t count {0};
 
-  while (reinterpret_cast<unsigned char*>(ap) < a + qgramvectorbytes) {
-    count += _mm_popcnt_u64(*ap++ ^ *bp++);
+  while (reinterpret_cast<unsigned char*>(qgram_vector_a_ptr) < qgram_a + qgramvectorbytes) {
+    count += _mm_popcnt_u64(*qgram_vector_a_ptr++ ^ *qgram_vector_b_ptr++);
   }
 
   return count;
