@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -24,5 +24,5 @@
 #include <cstdint>
 
 
-auto compareqgramvectors_popcnt(unsigned char * a, unsigned char * b)
+auto compareqgramvectors_popcnt(unsigned char * qgram_a, unsigned char * qgram_b)
   -> uint64_t;
