@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,17 +22,39 @@
 */
 
 #include "swarm.h"
-#include "threads.h"
-#include "util.h"
-
-#ifdef __x86_64__
-
-#include "popcnt.h"
+#include "utils/qgram_array.h"
+#include "utils/threads.h"
 
+#ifdef __aarch64__
+#include <arm_neon.h>
 #endif
 
 
+#ifdef __PPC__
+#ifdef __LITTLE_ENDIAN__
+#include <altivec.h>
+#else
+#error Big endian ppc64 CPUs not supported
+#endif
+#endif
 
+
+#ifdef __x86_64__
+
+#ifdef __SSE2__
+#include <emmintrin.h>  // SSE2 intrinsics
+#endif
+
+#include "popcnt.h"  // refactoring: fence with #ifdef __POPCNT__?
+
+#endif
+
+#include "utils/nt_codec.h"
+#include <cstdint>  // int64_t, uint64_t
+#include <cstring>  // memset
+
+
+qgramvector_t * qgrams {nullptr};
 static ThreadRunner * qgram_threads = nullptr;
 
 static struct thread_info_s
@@ -49,40 +71,40 @@ void findqgrams(unsigned char * seq, uint64_t seqlen,
 {
   /* set qgram bit vector by xoring occurrences of qgrams in sequence */
 
-  constexpr unsigned int max_range {7};
+  static constexpr unsigned int max_range {7};
 
-  memset(qgramvector, 0, qgramvectorbytes);
+  std::memset(qgramvector, 0, qgramvectorbytes);
 
   uint64_t qgram {0};
   unsigned int i {0};
 
-  while((i < qgramlength-1) && (i<seqlen))
+  while((i < qgramlength - 1) and (i < seqlen))
   {
-    qgram = (qgram << 2) | nt_extract(reinterpret_cast<char *>(seq), i);
-    i++;
+    qgram = (qgram << 2U) | nt_extract(reinterpret_cast<char *>(seq), i);
+    ++i;
   }
 
   while(i < seqlen)
   {
-    qgram = (qgram << 2) | nt_extract(reinterpret_cast<char *>(seq), i);
-    qgramvector[(qgram >> 3) & (qgramvectorbytes - 1)] ^= (1 << (qgram & max_range));
-    i++;
+    qgram = (qgram << 2U) | nt_extract(reinterpret_cast<char *>(seq), i);
+    qgramvector[(qgram >> 3U) & (qgramvectorbytes - 1)] ^= (1U << (qgram & max_range));
+    ++i;
   }
 }
 
 void qgram_work_diff(thread_info_s * tip);
 void qgram_worker(int64_t t);
-auto compareqgramvectors(unsigned char * a, unsigned char * b) -> uint64_t;
+auto compareqgramvectors(unsigned char * qgram_a, unsigned char * qgram_b) -> uint64_t;
 
 #ifdef __aarch64__
 
-uint64_t compareqgramvectors(unsigned char * a, unsigned char * b)
+uint64_t compareqgramvectors(unsigned char * qgram_a, unsigned char * qgram_b)
 {
-  uint8x16_t * ap = (uint8x16_t *) a;
-  uint8x16_t * bp = (uint8x16_t *) b;
+  uint8x16_t * ap = (uint8x16_t *) qgram_a;
+  uint8x16_t * bp = (uint8x16_t *) qgram_b;
   uint64_t count {0};
 
-  while ((unsigned char*)ap < a + qgramvectorbytes) {
+  while ((unsigned char*)ap < qgram_a + qgramvectorbytes) {
     count += vaddvq_u8(vcntq_u8(veorq_u8(*ap++, *bp++)));
   }
 
@@ -91,13 +113,13 @@ uint64_t compareqgramvectors(unsigned char * a, unsigned char * b)
 
 #elif defined __PPC__
 
-uint64_t compareqgramvectors(unsigned char * a, unsigned char * b)
+uint64_t compareqgramvectors(unsigned char * qgram_a, unsigned char * qgram_b)
 {
-  vector unsigned char * ap = (vector unsigned char *) a;
-  vector unsigned char * bp = (vector unsigned char *) b;
+  vector unsigned char * ap = (vector unsigned char *) qgram_a;
+  vector unsigned char * bp = (vector unsigned char *) qgram_b;
   vector unsigned long long count_vector = { 0, 0 };
 
-  while ((unsigned char *)ap < a + qgramvectorbytes) {
+  while ((unsigned char *)ap < qgram_a + qgramvectorbytes) {
     count_vector += vec_vpopcnt((vector unsigned long long)(vec_xor(*ap++, *bp++)));
   }
 
@@ -107,84 +129,82 @@ uint64_t compareqgramvectors(unsigned char * a, unsigned char * b)
 #elif defined __x86_64__
 
 auto popcount_128(__m128i x) -> uint64_t;
-auto compareqgramvectors_128(unsigned char * a, unsigned char * b) -> uint64_t;
+auto compareqgramvectors_128(unsigned char * qgram_a, unsigned char * qgram_b) -> uint64_t;
 
 auto popcount_128(__m128i x) -> uint64_t
 {
-  constexpr unsigned char m1 {0x55};  // '0101 0101'
-  constexpr unsigned char m2 {0x33};  // '0011 0011'
-  constexpr unsigned char m4 {0x0f};  // '0000 1111'
+  static constexpr unsigned char m1 {0x55};  // '0101 0101'
+  static constexpr unsigned char m2 {0x33};  // '0011 0011'
+  static constexpr unsigned char m4 {0x0f};  // '0000 1111'
 
-  __m128i mask1 = _mm_set_epi8(m1, m1, m1, m1, m1, m1, m1, m1,
-                               m1, m1, m1, m1, m1, m1, m1, m1);
+  const __m128i mask1 = _mm_set_epi8(m1, m1, m1, m1, m1, m1, m1, m1,
+                                     m1, m1, m1, m1, m1, m1, m1, m1);
 
-  __m128i mask2 = _mm_set_epi8(m2, m2, m2, m2, m2, m2, m2, m2,
-                               m2, m2, m2, m2, m2, m2, m2, m2);
+  const __m128i mask2 = _mm_set_epi8(m2, m2, m2, m2, m2, m2, m2, m2,
+                                     m2, m2, m2, m2, m2, m2, m2, m2);
 
-  __m128i mask4 = _mm_set_epi8(m4, m4, m4, m4, m4, m4, m4, m4,
-                               m4, m4, m4, m4, m4, m4, m4, m4);
+  const __m128i mask4 = _mm_set_epi8(m4, m4, m4, m4, m4, m4, m4, m4,
+                                     m4, m4, m4, m4, m4, m4, m4, m4);
 
-  __m128i zero = _mm_setzero_si128();
+  const __m128i zero = _mm_setzero_si128();
 
   /* add together 2 bits: 0+1, 2+3, 3+4, ... 126+127 */
 
-  __m128i a = _mm_srli_epi64(x, 1);
-  __m128i b = _mm_and_si128(x, mask1);
-  __m128i c = _mm_and_si128(a, mask1);
-  __m128i d = _mm_add_epi64(b, c);
+  const __m128i a = _mm_srli_epi64(x, 1);
+  const __m128i b = _mm_and_si128(x, mask1);
+  const __m128i c = _mm_and_si128(a, mask1);
+  const __m128i d = _mm_add_epi64(b, c);
 
   /* add together 4 bits: (0+1)+(2+3), ... (124+125)+(126+127) */
 
-  __m128i e = _mm_srli_epi64(d, 2);
-  __m128i f = _mm_and_si128(d, mask2);
-  __m128i g = _mm_and_si128(e, mask2);
-  __m128i h = _mm_add_epi64(f, g);
+  const __m128i e = _mm_srli_epi64(d, 2);
+  const __m128i f = _mm_and_si128(d, mask2);
+  const __m128i g = _mm_and_si128(e, mask2);
+  const __m128i h = _mm_add_epi64(f, g);
 
   /* add together 8 bits: (0..3)+(4..7), ... (120..123)+(124..127) */
 
-  __m128i i = _mm_srli_epi64(h, 4);
-  __m128i j = _mm_add_epi64(h, i);
-  __m128i k = _mm_and_si128(j, mask4);
+  const __m128i i = _mm_srli_epi64(h, 4);
+  const __m128i j = _mm_add_epi64(h, i);
+  const __m128i k = _mm_and_si128(j, mask4);
 
   /* add together 8 bytes: (0..63) and (64..127) */
 
-  __m128i l = _mm_sad_epu8(k, zero);
+  const __m128i l = _mm_sad_epu8(k, zero);
 
   /* add together 64-bit values into final 128 bit value */
 
-  __m128i m = _mm_srli_si128(l, 8);
-  __m128i n = _mm_add_epi64(m, l);
+  const __m128i m = _mm_srli_si128(l, 8);
+  const __m128i n = _mm_add_epi64(m, l);
 
   /* return low 64 bits: return value is always in range 0 to 128 */
 
-  auto o = reinterpret_cast<uint64_t>(_mm_movepi64_pi64(n));
-
-  return o;
+  return reinterpret_cast<uint64_t>(_mm_movepi64_pi64(n));
 }
 
-auto compareqgramvectors_128(unsigned char * a, unsigned char * b) -> uint64_t
+auto compareqgramvectors_128(unsigned char * qgram_a, unsigned char * qgram_b) -> uint64_t
 {
   /* Count number of different bits */
   /* Uses SSE2 but not POPCNT instruction */
   /* input MUST be 16-byte aligned */
 
-  auto * ap = reinterpret_cast<__m128i *>(a);
-  auto * bp = reinterpret_cast<__m128i *>(b);
+  auto * ap = reinterpret_cast<__m128i *>(qgram_a);
+  auto * bp = reinterpret_cast<__m128i *>(qgram_b);
   uint64_t count {0};
 
-  while (reinterpret_cast<unsigned char*>(ap) < a + qgramvectorbytes) {
+  while (reinterpret_cast<unsigned char*>(ap) < qgram_a + qgramvectorbytes) {
     count += popcount_128(_mm_xor_si128(*ap++, *bp++));
   }
 
   return count;
 }
 
-auto compareqgramvectors(unsigned char * a, unsigned char * b) -> uint64_t
+auto compareqgramvectors(unsigned char * qgram_a, unsigned char * qgram_b) -> uint64_t
 {
   if (popcnt_present != 0) {
-    return compareqgramvectors_popcnt(a, b);
+    return compareqgramvectors_popcnt(qgram_a, qgram_b);
   }
-  return compareqgramvectors_128(a, b);
+  return compareqgramvectors_128(qgram_a, qgram_b);
 }
 
 #else
@@ -198,20 +218,19 @@ inline auto db_getqgramvector(uint64_t seqno) -> unsigned char *
   return reinterpret_cast<unsigned char*>(qgrams + seqno);
 }
 
-inline auto qgram_diff(uint64_t a, uint64_t b) -> uint64_t
+inline auto qgram_diff(uint64_t seqno_a, uint64_t seqno_b) -> uint64_t
 {
-  uint64_t diffqgrams = compareqgramvectors(db_getqgramvector(a),
-                                            db_getqgramvector(b));
-  uint64_t mindiff = (diffqgrams + 2*qgramlength - 1)/(2*qgramlength);
-  return mindiff;
+  const uint64_t diffqgrams = compareqgramvectors(db_getqgramvector(seqno_a),
+                                                  db_getqgramvector(seqno_b));
+  return (diffqgrams + 2 * qgramlength - 1)/(2 * qgramlength);  // mindiff
 }
 
-void qgram_worker(int64_t t)
+auto qgram_worker(int64_t t) -> void
 {
   struct thread_info_s * tip = ti + t;
 
-  uint64_t seed = tip->seed;
-  uint64_t listlen = tip->listlen;
+  const uint64_t seed = tip->seed;
+  const uint64_t listlen = tip->listlen;
   uint64_t * amplist = tip->amplist;
   uint64_t * difflist = tip->difflist;
 
@@ -263,7 +282,7 @@ void qgram_diff_fast(uint64_t seed,
       for(auto t = 0ULL; t < thr; t++)
         {
           thread_info_s * tip = ti + t;
-          uint64_t chunk = (listrest + thrrest - 1) / thrrest;
+          const uint64_t chunk = (listrest + thrrest - 1) / thrrest;
 
           tip->seed = seed;
           tip->amplist = next_amplist;
@@ -273,7 +292,7 @@ void qgram_diff_fast(uint64_t seed,
           next_amplist += chunk;
           next_difflist += chunk;
           listrest -= chunk;
-          thrrest--;
+          --thrrest;
         }
 
       qgram_threads->run();
