@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,9 +23,10 @@
 
 #include <cstdint>
 
+
 void findqgrams(unsigned char * seq, uint64_t seqlen,
                 unsigned char * qgramvector);
-auto qgram_diff(uint64_t a, uint64_t b) -> uint64_t;
+auto qgram_diff(uint64_t seqno_a, uint64_t seqno_b) -> uint64_t;
 void qgram_diff_fast(uint64_t seed,
                      uint64_t listlen,
                      uint64_t * amplist,
