@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -25,14 +25,20 @@
 #include "db.h"
 #include "search8.h"
 #include "search16.h"
-#include "threads.h"
+#include "utils/nt_codec.h"
+#include "utils/score_matrix.h"
+#include "utils/threads.h"
+#include <cassert>  // assert()
+#include <climits>
+#include <cstdint>  // int64_t, uint64_t
+#include <pthread.h>
 
 
 static pthread_mutex_t scan_mutex;
 
 static ThreadRunner * search_threads = nullptr;
 
-struct search_data
+struct Search_data
 {
   BYTE ** qtable;
   WORD ** qtable_w;
@@ -48,7 +54,7 @@ struct search_data
   uint64_t target_index;
 };
 
-static struct search_data * sd;
+static struct Search_data * search_data;
 static uint64_t master_next;
 static uint64_t master_length;
 static uint64_t remainingchunks;
@@ -57,48 +63,46 @@ static uint64_t * master_scores;
 static uint64_t * master_diffs;
 static uint64_t * master_alignlengths;
 static int master_bits;
-static uint64_t dirbufferbytes;
+static uint64_t dirbuffersize;
 
 queryinfo_t query;
 uint64_t longestdbsequence;
 
 
-void search_alloc(struct search_data * sdp)
+void search_alloc(struct Search_data * search_data)
 {
-  constexpr unsigned int one_kilobyte {1024};
-  constexpr unsigned int nt_per_uint64 {32};
-  constexpr unsigned int bytes_per_uint64 {8};
+  static constexpr unsigned int one_kilobyte {1024};
+  static constexpr unsigned int nt_per_uint64 {32};
 
-  dirbufferbytes =
-    bytes_per_uint64 * longestdbsequence * ((longestdbsequence + 3) / 4) * 4;
-  sdp->qtable = new BYTE*[longestdbsequence];
-  sdp->qtable_w = new WORD*[longestdbsequence];
-  sdp->dprofile = new BYTE[2 * one_kilobyte];  // 4 * 16 * 32
-  sdp->dprofile_w = new WORD[2 * one_kilobyte];  // 4 * 2 * 8 * 32
-  sdp->hearray = new BYTE[longestdbsequence * nt_per_uint64] { };
-  sdp->dir_array = new uint64_t[dirbufferbytes] { };
+  dirbuffersize = longestdbsequence * ((longestdbsequence + 3) / 4) * 4;
+  search_data->qtable = new BYTE*[longestdbsequence];
+  search_data->qtable_w = new WORD*[longestdbsequence];
+  search_data->dprofile = new BYTE[2 * one_kilobyte];  // 4 * 16 * 32
+  search_data->dprofile_w = new WORD[1 * one_kilobyte];  // 4 * 2 * 8 * 32
+  search_data->hearray = new BYTE[longestdbsequence * nt_per_uint64] { };
+  search_data->dir_array = new uint64_t[dirbuffersize] { };
 }
 
-void search_free(struct search_data * sdp)
+void search_free(struct Search_data * search_data)
 {
-  delete [] sdp->qtable;
-  sdp->qtable = nullptr;
-  delete [] sdp->qtable_w;
-  sdp->qtable_w = nullptr;
-  delete [] sdp->dprofile;
-  sdp->dprofile = nullptr;
-  delete [] sdp->dprofile_w;
-  sdp->dprofile_w = nullptr;
-  delete [] sdp->hearray;
-  sdp->hearray = nullptr;
-  delete [] sdp->dir_array;
-  sdp->dir_array = nullptr;
+  delete [] search_data->qtable;
+  search_data->qtable = nullptr;
+  delete [] search_data->qtable_w;
+  search_data->qtable_w = nullptr;
+  delete [] search_data->dprofile;
+  search_data->dprofile = nullptr;
+  delete [] search_data->dprofile_w;
+  search_data->dprofile_w = nullptr;
+  delete [] search_data->hearray;
+  search_data->hearray = nullptr;
+  delete [] search_data->dir_array;
+  search_data->dir_array = nullptr;
 }
 
-void search_init(struct search_data * sdp)
+void search_init(struct Search_data * search_data)
 {
-  constexpr int byte_multiplier {64};
-  constexpr int word_multiplier {32};
+  static constexpr int byte_multiplier {64};
+  static constexpr int word_multiplier {32};
 
   for(auto i = 0U; i < query.len; i++)
   {
@@ -106,51 +110,54 @@ void search_init(struct search_data * sdp)
     const int byte_offset {byte_multiplier * nt_value};  // 1, 64, 128, or 192
     const int word_offset {word_multiplier * nt_value};  // 1, 32,  64, or 128
 
-    sdp->qtable[i]   = sdp->dprofile   + byte_offset;
-    sdp->qtable_w[i] = sdp->dprofile_w + word_offset;
+    search_data->qtable[i]   = search_data->dprofile   + byte_offset;
+    search_data->qtable_w[i] = search_data->dprofile_w + word_offset;
   }
 }
 
-void search_chunk(struct search_data * sdp, const int64_t bits)
+void search_chunk(struct Search_data * search_data, const int64_t bits)
 {
-  constexpr unsigned int bit_mode_16 {16};
+  static auto score_matrix_8 = create_score_matrix<unsigned char>(penalty_mismatch);
+  static auto score_matrix_16 = create_score_matrix<unsigned short>(penalty_mismatch);
 
-  assert(sdp->target_count != 0);
-  assert((bits == bit_mode_16) || (bits == bit_mode_16 / 2));
+  static constexpr unsigned int bit_mode_16 {16};
+
+  assert(search_data->target_count != 0);
+  assert((bits == bit_mode_16) or (bits == bit_mode_16 / 2));
 
  if (bits == bit_mode_16)
    {
-    search16(sdp->qtable_w,
+    search16(search_data->qtable_w,
              static_cast<WORD>(penalty_gapopen),
              static_cast<WORD>(penalty_gapextend),
-             static_cast<WORD*>(score_matrix_16),
-             sdp->dprofile_w,
-             reinterpret_cast<WORD*>(sdp->hearray),
-             sdp->target_count,
-             master_targets + sdp->target_index,
-             master_scores + sdp->target_index,
-             master_diffs + sdp->target_index,
-             master_alignlengths + sdp->target_index,
+             static_cast<WORD*>(score_matrix_16.data()),
+             search_data->dprofile_w,
+             reinterpret_cast<WORD*>(search_data->hearray),
+             search_data->target_count,
+             master_targets + search_data->target_index,
+             master_scores + search_data->target_index,
+             master_diffs + search_data->target_index,
+             master_alignlengths + search_data->target_index,
              static_cast<uint64_t>(query.len),
-             dirbufferbytes / sizeof(uint64_t),
-             sdp->dir_array,
+             dirbuffersize,
+             search_data->dir_array,
              longestdbsequence);
    }
  else {
-    search8(sdp->qtable,
+    search8(search_data->qtable,
             static_cast<BYTE>(penalty_gapopen),
             static_cast<BYTE>(penalty_gapextend),
-            static_cast<BYTE*>(score_matrix_8),
-            sdp->dprofile,
-            sdp->hearray,
-            sdp->target_count,
-            master_targets + sdp->target_index,
-            master_scores + sdp->target_index,
-            master_diffs + sdp->target_index,
-            master_alignlengths + sdp->target_index,
+            static_cast<BYTE*>(score_matrix_8.data()),
+            search_data->dprofile,
+            search_data->hearray,
+            search_data->target_count,
+            master_targets + search_data->target_index,
+            master_scores + search_data->target_index,
+            master_diffs + search_data->target_index,
+            master_alignlengths + search_data->target_index,
             static_cast<uint64_t>(query.len),
-            dirbufferbytes / sizeof(uint64_t),
-            sdp->dir_array,
+            dirbuffersize,
+            search_data->dir_array,
             longestdbsequence);
  }
 }
@@ -173,7 +180,7 @@ auto search_getwork(uint64_t * countref, uint64_t * firstref) -> bool
       * firstref = master_next;
 
       master_next += chunksize;
-      remainingchunks--;
+      --remainingchunks;
       status = true;
     }
 
@@ -183,11 +190,11 @@ auto search_getwork(uint64_t * countref, uint64_t * firstref) -> bool
 }
 
 
-void search_worker_core(const int64_t t)
+void search_worker_core(const int64_t thread_id)
 {
-  search_init(sd + t);
-  while(search_getwork(& sd[t].target_count, & sd[t].target_index)) {
-    search_chunk(sd + t, master_bits);
+  search_init(search_data + thread_id);
+  while(search_getwork(& search_data[thread_id].target_count, & search_data[thread_id].target_index)) {
+    search_chunk(search_data + thread_id, master_bits);
   }
 }
 
@@ -195,15 +202,15 @@ void search_worker_core(const int64_t t)
 auto adjust_thread_number(const int n_bits,
                           const uint64_t remaining_sequences,
                           uint64_t n_threads) -> uint64_t {
-  constexpr unsigned int channels_8 {8};
-  constexpr unsigned int channels_16 {16};
-  constexpr unsigned int bit_mode_16 {16};  // refactoring: should be an enum class
-  const auto channels { (n_bits == bit_mode_16) ? channels_8 : channels_16 };
+  static constexpr unsigned int channels_8 {8};
+  static constexpr unsigned int channels_16 {16};
+  static constexpr unsigned int bit_mode_16 {16};  // refactoring: should be an enum class
+  const auto channels = (n_bits == bit_mode_16) ? channels_8 : channels_16;
 
   assert(remaining_sequences != 0);
   assert(n_threads != 0);
   assert((n_threads - 1) <= (ULLONG_MAX / channels_8));
-  assert((n_bits == bit_mode_16) || (n_bits == bit_mode_16 / 2));
+  assert((n_bits == bit_mode_16) or (n_bits == bit_mode_16 / 2));
 
   while (remaining_sequences <= (n_threads - 1) * channels) {
     --n_threads;
@@ -249,11 +256,10 @@ void search_do(uint64_t query_no,
   master_alignlengths = alignlengths;
   master_bits = bits;
 
-  const auto thr {
+  const auto thr =
     adjust_thread_number(bits,
                          master_length,
-                         static_cast<uint64_t>(opt_threads))
-  };
+                         static_cast<uint64_t>(opt_threads));
 
   remainingchunks = thr;
 
@@ -269,10 +275,10 @@ void search_begin()
 {
   longestdbsequence = db_getlongestsequence();
 
-  sd = new struct search_data[static_cast<uint64_t>(opt_threads)];
+  search_data = new struct Search_data[static_cast<uint64_t>(opt_threads)];
 
-  for(auto t = 0LL; t < opt_threads; t++) {
-    search_alloc(sd + t);
+  for(auto thread_id = 0LL; thread_id < opt_threads; thread_id++) {
+    search_alloc(search_data + thread_id);
   }
 
   pthread_mutex_init(& scan_mutex, nullptr);
@@ -292,9 +298,9 @@ void search_end()
 
   pthread_mutex_destroy(& scan_mutex);
 
-  for(auto t = 0LL; t < opt_threads; t++) {
-    search_free(sd + t);
+  for(auto thread_id = 0LL; thread_id < opt_threads; thread_id++) {
+    search_free(search_data + thread_id);
   }
-  delete [] sd;
-  sd = nullptr;
+  delete [] search_data;
+  search_data = nullptr;
 }
