@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,14 +23,41 @@
 
 #include "swarm.h"
 #include "db.h"
+#include <array>
 
 #ifdef __x86_64__
 
+#ifdef __SSE2__
+#include <emmintrin.h>  // SSE2 intrinsics
+#endif
+
 #include "ssse3.h"
 #include "sse41.h"
 
 #endif
 
+#include "utils/nt_codec.h"
+#include <cstdint>  // int64_t, uint64_t
+
+
+#ifdef __aarch64__
+#include <arm_neon.h>
+#elif defined __x86_64__
+
+#elif defined __PPC__
+
+#ifdef __LITTLE_ENDIAN__
+#include <altivec.h>
+#else
+#error Big endian ppc64 CPUs not supported
+#endif
+
+#else
+
+#error Unknown architecture
+#endif
+
+
 constexpr unsigned int channels {8};
 constexpr unsigned int cdepth {4};
 using BYTE = unsigned char;
@@ -151,7 +178,7 @@ inline void dprofile_fill16(WORD * dprofile_word,
                             WORD * score_matrix_word,
                             BYTE * dseq)
 {
-  constexpr unsigned int multiplier {5};
+  static constexpr unsigned int multiplier {5};
   VECTORTYPE reg0;
   VECTORTYPE reg1;
   VECTORTYPE reg2;
@@ -187,7 +214,7 @@ inline void dprofile_fill16(WORD * dprofile_word,
 
   for(auto j = 0U; j < cdepth; j++)
     {
-      unsigned int d[channels];
+      std::array<unsigned int, channels> d {{}};
       for(auto z = 0U; z < channels; z++) {
         d[z] = (static_cast<unsigned int>(dseq[j * channels + z])) << multiplier;
       }
@@ -240,9 +267,6 @@ inline void dprofile_fill16(WORD * dprofile_word,
           v_store(dprofile_word + cdepth * channels * (i + 7) + channels * j, reg31);
         }
     }
-#if 0
-  dprofile_dump16(dprofile_word);
-#endif
 }
 
 inline void onestep_16(VECTORTYPE & H,
@@ -282,11 +306,11 @@ void align_cells_regular_16(VECTORTYPE * Sm,
                             uint64_t * dir_long,
                             VECTORTYPE * H0)
 {
-  constexpr unsigned int step {16};
-  constexpr unsigned int offset0 {0};
-  constexpr unsigned int offset1 {offset0 + 4};
-  constexpr unsigned int offset2 {offset1 + 4};
-  constexpr unsigned int offset3 {offset2 + 4};
+  static constexpr unsigned int step {16};
+  static constexpr unsigned int offset0 {0};
+  static constexpr unsigned int offset1 {offset0 + 4};
+  static constexpr unsigned int offset2 {offset1 + 4};
+  static constexpr unsigned int offset3 {offset2 + 4};
 
   VECTORTYPE Q;
   VECTORTYPE R;
@@ -364,11 +388,11 @@ void align_cells_masked_16(VECTORTYPE * Sm,
                            VECTORTYPE * MR,
                            VECTORTYPE * MQ0)
 {
-  constexpr unsigned int step {16};
-  constexpr unsigned int offset0 {0};
-  constexpr unsigned int offset1 {offset0 + 4};
-  constexpr unsigned int offset2 {offset1 + 4};
-  constexpr unsigned int offset3 {offset2 + 4};
+  static constexpr unsigned int step {16};
+  static constexpr unsigned int offset0 {0};
+  static constexpr unsigned int offset1 {offset0 + 4};
+  static constexpr unsigned int offset2 {offset1 + 4};
+  static constexpr unsigned int offset3 {offset2 + 4};
 
   VECTORTYPE Q;
   VECTORTYPE R;
@@ -457,74 +481,74 @@ auto backtrack_16(char * qseq,
                   uint64_t * alignmentlengthp,
                   const uint64_t longestdbsequence) -> uint64_t
 {
-  constexpr unsigned int offset0 {0};
-  constexpr unsigned int offset1 {offset0 + 16};
-  constexpr unsigned int offset2 {offset1 + 16};
-  constexpr unsigned int offset3 {offset2 + 16};
-  uint64_t maskup      = 3ULL << (2 * channel + offset0);
-  uint64_t maskleft    = 3ULL << (2 * channel + offset1);
-  uint64_t maskextup   = 3ULL << (2 * channel + offset2);
-  uint64_t maskextleft = 3ULL << (2 * channel + offset3);
+  static constexpr unsigned int offset0 {0};
+  static constexpr unsigned int offset1 {offset0 + 16};
+  static constexpr unsigned int offset2 {offset1 + 16};
+  static constexpr unsigned int offset3 {offset2 + 16};
+  const uint64_t maskup      = 3ULL << (2 * channel + offset0);
+  const uint64_t maskleft    = 3ULL << (2 * channel + offset1);
+  const uint64_t maskextup   = 3ULL << (2 * channel + offset2);
+  const uint64_t maskextleft = 3ULL << (2 * channel + offset3);
 
-  int64_t i = static_cast<int64_t>(qlen) - 1;
-  int64_t j = static_cast<int64_t>(dlen) - 1;
+  auto i = static_cast<int64_t>(qlen) - 1;
+  auto j = static_cast<int64_t>(dlen) - 1;
   uint64_t aligned {0};
   uint64_t matches {0};
-  char op {0};
+  char operation {0};  // Insertion, Deletion or Match  // refactoring to enum class?
 
 #undef SHOWALIGNMENT
 #ifdef SHOWALIGNMENT
   printf("alignment, reversed: ");
 #endif
 
-  while ((i >= 0) && (j >= 0))
+  while ((i >= 0) and (j >= 0))
     {
-      aligned++;
+      ++aligned;
 
-      uint64_t d
+      const uint64_t d
         = dirbuffer[(offset
                      + longestdbsequence * 4 * static_cast<uint64_t>(j / 4)
-                     + static_cast<uint64_t>(4 * i + (j & 3)))
-                    % dirbuffersize];
+                     + static_cast<uint64_t>(4 * i + (static_cast<uint64_t>(j) & 3U)))
+                    % dirbuffersize];  // refactoring: how to rename that variable?
 
-      if ((op == 'I') && ((d & maskextleft) == 0U))
+      if ((operation == 'I') and ((d & maskextleft) == 0U))
         {
-          j--;
+          --j;
         }
-      else if ((op == 'D') && ((d & maskextup) == 0U))
+      else if ((operation == 'D') and ((d & maskextup) == 0U))
         {
-          i--;
+          --i;
         }
       else if ((d & maskleft) != 0U)
         {
-          j--;
-          op = 'I';
+          --j;
+          operation = 'I';
         }
       else if ((d & maskup) == 0U)
         {
-          i--;
-          op = 'D';
+          --i;
+          operation = 'D';
         }
       else
         {
           if (nt_extract(qseq, static_cast<uint64_t>(i)) ==
               nt_extract(dseq, static_cast<uint64_t>(j))) {
-            matches++;
+            ++matches;
           }
-          i--;
-          j--;
-          op = 'M';
+          --i;
+          --j;
+          operation = 'M';
         }
 
 #ifdef SHOWALIGNMENT
-      printf("%c", op);
+      printf("%c", operation);
 #endif
     }
 
   while (i >= 0)
     {
-      aligned++;
-      i--;
+      ++aligned;
+      --i;
 #ifdef SHOWALIGNMENT
       printf("D");
 #endif
@@ -532,8 +556,8 @@ auto backtrack_16(char * qseq,
 
   while (j >= 0)
     {
-      aligned++;
-      j--;
+      ++aligned;
+      --j;
 #ifdef SHOWALIGNMENT
       printf("I");
 #endif
@@ -574,10 +598,10 @@ void search16(WORD * * q_start,
   VECTORTYPE *hep {nullptr};
   VECTORTYPE **qp {nullptr};
 
-  uint64_t d_pos[channels];
-  uint64_t d_offset[channels];
-  char * d_address[channels];
-  uint64_t d_length[channels];
+  std::array<uint64_t, channels> d_pos {{}};
+  std::array<uint64_t, channels> d_offset {{}};
+  std::array<char *, channels> d_address {{}};
+  std::array<uint64_t, channels> d_length {{}};
 
   VECTORTYPE dseqalloc[cdepth];
 
@@ -587,7 +611,7 @@ void search16(WORD * * q_start,
 
   BYTE * dseq = reinterpret_cast<BYTE*>(& dseqalloc);
 
-  int64_t seq_id[channels];
+  std::array<int64_t, channels> seq_id {{}};
   uint64_t next_id {0};
   uint64_t done {0};
 
@@ -609,6 +633,7 @@ void search16(WORD * * q_start,
   hep = CAST_VECTOR_p(hearray);
   qp = reinterpret_cast<VECTORTYPE**>(q_start);
 
+  // refactoring: initialize arrays
   for(auto c = 0U; c < channels; c++)
     {
       d_address[c] = nullptr;
@@ -706,16 +731,16 @@ void search16(WORD * * q_start,
 
                   M = v_xor(M, T);
 
-                  int64_t cand_id = seq_id[c];
+                  const int64_t cand_id = seq_id[c];
 
                   if (cand_id >= 0)
                     {
                       // save score
 
                       char * dbseq = reinterpret_cast<char*>(d_address[c]);
-                      uint64_t dbseqlen = d_length[c];
-                      uint64_t z = (dbseqlen+3) % 4;
-                      uint64_t score
+                      const uint64_t dbseqlen = d_length[c];
+                      const uint64_t z = (dbseqlen + 3) % 4;
+                      const uint64_t score
                         = (reinterpret_cast<WORD*>(S))[z * channels + c];
                       scores[cand_id] = score;
 
@@ -723,7 +748,7 @@ void search16(WORD * * q_start,
 
                       if (score < UINT16_MAX)
                         {
-                          uint64_t offset = d_offset[c];
+                          const uint64_t offset = d_offset[c];
                           diff = backtrack_16(query.seq, dbseq, qlen, dbseqlen,
                                               dirbuffer,
                                               offset,
@@ -738,14 +763,14 @@ void search16(WORD * * q_start,
 
                       diffs[cand_id] = diff;
 
-                      done++;
+                      ++done;
                     }
 
                   if (next_id < sequences)
                     {
                       // get next sequence
                       seq_id[c] = static_cast<int64_t>(next_id);
-                      uint64_t seqno = seqnos[next_id];
+                      const uint64_t seqno = seqnos[next_id];
                       char * address {nullptr};
                       unsigned int length {0};
 
@@ -756,7 +781,7 @@ void search16(WORD * * q_start,
 
                       d_pos[c] = 0;
                       d_offset[c] = static_cast<uint64_t>(dir - dirbuffer);
-                      next_id++;
+                      ++next_id;
 
                       (reinterpret_cast<WORD*>(&H0))[c] = 0;
                       (reinterpret_cast<WORD*>(&F0))[c] = 2 * gap_open_penalty + 2 * gap_extend_penalty;
