@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,9 +22,46 @@
 */
 
 #include "swarm.h"
+#include "db.h"
+#include <array>
+
+// refactoring: C++26 std::simd
+#ifdef __x86_64__
+
+#ifdef __SSE2__
+#include <emmintrin.h>  // SSE2 intrinsics
+#endif
+
+#include "ssse3.h"
+
+#endif
+
+#include "utils/nt_codec.h"
+#include <cstdint>  // int64_t, uint64_t
+
+
+#ifdef __aarch64__
+#include <arm_neon.h>
+#elif defined __x86_64__
+
+#elif defined __PPC__
+
+#ifdef __LITTLE_ENDIAN__
+#include <altivec.h>
+#else
+#error Big endian ppc64 CPUs not supported
+#endif
+
+#else
+
+#error Unknown architecture
+#endif
+
 
 constexpr unsigned int channels {16};
 constexpr unsigned int cdepth {4};
+using BYTE = unsigned char;
+
 
 /* uses 16 unsigned 8-bit values */
 
@@ -144,35 +181,12 @@ const vector unsigned char perm_bits =
 
 #endif
 
-void align_cells_regular_8(VECTORTYPE * Sm,
-                           VECTORTYPE * hep,
-                           VECTORTYPE ** qp,
-                           VECTORTYPE * Qm,
-                           VECTORTYPE * Rm,
-                           uint64_t ql,
-                           VECTORTYPE * F0,
-                           uint64_t * dir_long,
-                           VECTORTYPE * H0);
-
-void align_cells_masked_8(VECTORTYPE * Sm,
-                          VECTORTYPE * hep,
-                          VECTORTYPE ** qp,
-                          VECTORTYPE * Qm,
-                          VECTORTYPE * Rm,
-                          uint64_t ql,
-                          VECTORTYPE * F0,
-                          uint64_t * dir_long,
-                          VECTORTYPE * H0,
-                          VECTORTYPE * Mm,
-                          VECTORTYPE * MQ,
-                          VECTORTYPE * MR,
-                          VECTORTYPE * MQ0);
 
 inline void dprofile_fill8(BYTE * dprofile,
                            BYTE * score_matrix,
                            BYTE * dseq)
 {
-  constexpr unsigned int multiplier {5};
+  static constexpr unsigned int multiplier {5};
   VECTORTYPE reg0;
   VECTORTYPE reg1;
   VECTORTYPE reg2;
@@ -192,7 +206,7 @@ inline void dprofile_fill8(BYTE * dprofile,
 
   for(auto j = 0U; j < cdepth; j++)
     {
-      unsigned int d[channels];
+      std::array<unsigned int, channels> d {{}};
       for(auto i = 0U; i < channels; i++) {
         d[i] = (static_cast<unsigned int>(dseq[j * channels + i])) << multiplier;
       }
@@ -475,7 +489,7 @@ inline void dprofile_fill8(BYTE * dprofile,
       reg6  = v_merge_lo_64(reg6, reg14);
       reg15 = v_merge_hi_64(reg15, reg14);
 
-      v_store(dprofile + 16 * j + 1536 +   0, reg0);
+      v_store(dprofile + 16 * j + 1536 +   0, reg0);  // refactoring: 1536 = 6 * 256 bits?
       v_store(dprofile + 16 * j + 1536 +  64, reg3);
       v_store(dprofile + 16 * j + 1536 + 128, reg2);
       v_store(dprofile + 16 * j + 1536 + 192, reg7);
@@ -484,9 +498,6 @@ inline void dprofile_fill8(BYTE * dprofile,
       v_store(dprofile + 16 * j + 1536 + 384, reg6);
       v_store(dprofile + 16 * j + 1536 + 448, reg15);
     }
-#if 0
-  dprofile_dump8(dprofile);
-#endif
 }
 
 inline void onestep_8(VECTORTYPE & H,
@@ -516,6 +527,7 @@ inline void onestep_8(VECTORTYPE & H,
   *((DIR) + 3) = v_mask_eq(H, E);
 }
 
+
 void align_cells_regular_8(VECTORTYPE * Sm,
                            VECTORTYPE * hep,
                            VECTORTYPE ** qp,
@@ -566,10 +578,10 @@ void align_cells_regular_8(VECTORTYPE * Sm,
   for(auto i = 0ULL; i < ql; i++)
     {
       VECTORTYPE *x {nullptr};
-      constexpr unsigned int j0 {0};
-      constexpr unsigned int j1 {j0 + 4};
-      constexpr unsigned int j2 {j1 + 4};
-      constexpr unsigned int j3 {j2 + 4};
+      static constexpr unsigned int j0 {0};
+      static constexpr unsigned int j1 {j0 + 4};
+      static constexpr unsigned int j2 {j1 + 4};
+      static constexpr unsigned int j3 {j2 + 4};
 
       x = qp[i + 0];
       h4 = hep[2 * i + 0];
@@ -592,6 +604,7 @@ void align_cells_regular_8(VECTORTYPE * Sm,
   Sm[3] = h8;
 }
 
+
 void align_cells_masked_8(VECTORTYPE * Sm,
                           VECTORTYPE * hep,
                           VECTORTYPE ** qp,
@@ -645,10 +658,10 @@ void align_cells_masked_8(VECTORTYPE * Sm,
 
   for(auto i = 0ULL; i < ql; i++)
     {
-      constexpr unsigned int j0 {0};
-      constexpr unsigned int j1 {j0 + 4};
-      constexpr unsigned int j2 {j1 + 4};
-      constexpr unsigned int j3 {j2 + 4};
+      static constexpr unsigned int j0 {0};
+      static constexpr unsigned int j1 {j0 + 4};
+      static constexpr unsigned int j2 {j1 + 4};
+      static constexpr unsigned int j3 {j2 + 4};
 
       VECTORTYPE * x {nullptr};
 
@@ -687,140 +700,86 @@ void align_cells_masked_8(VECTORTYPE * Sm,
   Sm[3] = h8;
 }
 
+
 inline auto backtrack_8(char * qseq,
-                            char * dseq,
-                            uint64_t qlen,
-                            uint64_t dlen,
-                            uint64_t * dirbuffer,
-                            uint64_t offset,
-                            uint64_t dirbuffersize,
-                            uint64_t channel,
-                            uint64_t * alignmentlengthp) -> uint64_t
+                        char * dseq,
+                        uint64_t qlen,
+                        uint64_t dlen,
+                        uint64_t * dirbuffer,
+                        uint64_t offset,
+                        uint64_t dirbuffersize,
+                        uint64_t channel,
+                        uint64_t * alignmentlengthp,
+                        const uint64_t longestdbsequence) -> uint64_t
 {
-  constexpr unsigned int offset0 {0};
-  constexpr unsigned int offset1 {offset0 + 16};
-  constexpr unsigned int offset2 {offset1 + 16};
-  constexpr unsigned int offset3 {offset2 + 16};
+  static constexpr unsigned int offset0 {0};
+  static constexpr unsigned int offset1 {offset0 + 16};
+  static constexpr unsigned int offset2 {offset1 + 16};
+  static constexpr unsigned int offset3 {offset2 + 16};
+  const uint64_t maskup      = 1ULL << (channel + offset0);
+  const uint64_t maskleft    = 1ULL << (channel + offset1);
+  const uint64_t maskextup   = 1ULL << (channel + offset2);
+  const uint64_t maskextleft = 1ULL << (channel + offset3);
 
-  uint64_t maskup      = 1ULL << (channel + offset0);
-  uint64_t maskleft    = 1ULL << (channel + offset1);
-  uint64_t maskextup   = 1ULL << (channel + offset2);
-  uint64_t maskextleft = 1ULL << (channel + offset3);
-
-#if 0
-
-  printf("Dumping backtracking array\n");
-
-  for(auto i = 0ULL; i < qlen; i++)
-    {
-      for(auto j = 0ULL; j < dlen; j++)
-        {
-          uint64_t d = dirbuffer[(offset + longestdbsequence * 4 * (j / 4)
-                                  + 4 * i + (j & 3)) % dirbuffersize];
-          if (d & maskleft)
-            {
-              printf("<");
-            }
-          else if (!(d & maskup))
-            {
-              printf("^");
-            }
-          else
-            {
-              printf("\\");
-            }
-        }
-      printf("\n");
-    }
-
-  printf("Dumping gap extension array\n");
-
-  for(auto i = 0ULL; i < qlen; i++)
-    {
-      for(auto j = 0ULL; j < dlen; j++)
-        {
-          uint64_t d = dirbuffer[(offset + longestdbsequence * 4 * (j / 4)
-                                  + 4 * i + (j & 3)) % dirbuffersize];
-          if (!(d & maskextup))
-            {
-              if (!(d & maskextleft))
-                printf("+");
-              else
-                printf("^");
-            }
-          else if (!(d & maskextleft))
-            {
-              printf("<");
-            }
-          else
-            {
-              printf("\\");
-            }
-        }
-      printf("\n");
-    }
-
-#endif
-
-  int64_t i = static_cast<int64_t>(qlen) - 1;
-  int64_t j = static_cast<int64_t>(dlen) - 1;
+  auto i = static_cast<int64_t>(qlen) - 1;
+  auto j = static_cast<int64_t>(dlen) - 1;
   uint64_t aligned {0};
   uint64_t matches {0};
-  char op {0};
+  char operation {0};  // Insertion, Deletion or Match  // refactoring to enum class?
 
 #undef SHOWALIGNMENT
 #ifdef SHOWALIGNMENT
   printf("alignment, reversed: ");
 #endif
 
-  while ((i >= 0) && (j >= 0))
+  while ((i >= 0) and (j >= 0))
     {
-      aligned++;
+      ++aligned;
 
-      uint64_t d
+      const uint64_t d
         = dirbuffer[(offset
                      + longestdbsequence * 4 * static_cast<uint64_t>(j / 4)
-                     + static_cast<uint64_t>(4 * i + (j & 3)))
-                    % dirbuffersize];
+                     + static_cast<uint64_t>(4 * i + (static_cast<uint64_t>(j) & 3U)))
+                    % dirbuffersize];  // refactoring: how to rename that variable?
 
-      if ((op == 'I') && ((d & maskextleft) == 0U))
+      if ((operation == 'I') and ((d & maskextleft) == 0U))
         {
-          j--;
+          --j;
         }
-      else if ((op == 'D') && ((d & maskextup) == 0U))
+      else if ((operation == 'D') and ((d & maskextup) == 0U))
         {
-          i--;
+          --i;
         }
       else if ((d & maskleft) != 0U)
         {
-          j--;
-          op = 'I';
+          --j;
+          operation = 'I';
         }
       else if ((d & maskup) == 0U)
         {
-          i--;
-          op = 'D';
+          --i;
+          operation = 'D';
         }
       else
         {
           if (nt_extract(qseq, static_cast<uint64_t>(i)) ==
               nt_extract(dseq, static_cast<uint64_t>(j))) {
-            matches++;
+            ++matches;
           }
-          i--;
-          j--;
-          op = 'M';
+          --i;
+          --j;
+          operation = 'M';
         }
 
 #ifdef SHOWALIGNMENT
-      printf("%c", op);
+      printf("%c", operation);
 #endif
     }
 
   while (i >= 0)
     {
-      aligned++;
-      i--;
+      ++aligned;
+      --i;
 #ifdef SHOWALIGNMENT
       printf("D");
 #endif
@@ -828,8 +787,8 @@ inline auto backtrack_8(char * qseq,
 
   while (j >= 0)
     {
-      aligned++;
-      j--;
+      ++aligned;
+      --j;
 #ifdef SHOWALIGNMENT
       printf("I");
 #endif
@@ -843,6 +802,7 @@ inline auto backtrack_8(char * qseq,
   return aligned - matches;
 }
 
+
 void search8(BYTE * * q_start,
              BYTE gap_open_penalty,
              BYTE gap_extend_penalty,
@@ -856,7 +816,8 @@ void search8(BYTE * * q_start,
              uint64_t * alignmentlengths,
              uint64_t qlen,
              uint64_t dirbuffersize,
-             uint64_t * dirbuffer)
+             uint64_t * dirbuffer,
+             const uint64_t longestdbsequence)
 {
   VECTORTYPE Q;
   VECTORTYPE R;
@@ -869,10 +830,10 @@ void search8(BYTE * * q_start,
   VECTORTYPE *hep {nullptr};
   VECTORTYPE **qp {nullptr};
 
-  uint64_t d_pos[channels];
-  uint64_t d_offset[channels];
-  char * d_address[channels];
-  uint64_t d_length[channels];
+  std::array<uint64_t, channels> d_pos {{}};
+  std::array<uint64_t, channels> d_offset {{}};
+  std::array<char *, channels> d_address {{}};
+  std::array<uint64_t, channels> d_length {{}};
 
   VECTORTYPE dseqalloc[cdepth];
 
@@ -882,7 +843,7 @@ void search8(BYTE * * q_start,
 
   BYTE * dseq = reinterpret_cast<BYTE*>(& dseqalloc);
 
-  int64_t seq_id[channels];
+  std::array<int64_t, channels> seq_id {{}};
   uint64_t next_id {0};
   uint64_t done {0};
 
@@ -907,6 +868,7 @@ void search8(BYTE * * q_start,
   hep = CAST_VECTOR_p(hearray);
   qp = reinterpret_cast<VECTORTYPE**>(q_start);
 
+  // refactoring: initialize arrays
   for(auto c = 0U; c < channels; c++)
     {
       d_address[c] = nullptr;
@@ -994,16 +956,16 @@ void search8(BYTE * * q_start,
 
                   M = v_xor(M, T);
 
-                  int64_t cand_id = seq_id[c];
+                  const int64_t cand_id = seq_id[c];
 
                   if (cand_id >= 0)
                     {
                       // save score
 
                       char * dbseq = reinterpret_cast<char*>(d_address[c]);
-                      uint64_t dbseqlen = d_length[c];
-                      uint64_t z = (dbseqlen + 3) % 4;
-                      uint64_t score
+                      const uint64_t dbseqlen = d_length[c];
+                      const uint64_t z = (dbseqlen + 3) % 4;
+                      const uint64_t score
                         = (reinterpret_cast<BYTE*>(S))[z * channels + c];
                       scores[cand_id] = score;
 
@@ -1011,12 +973,13 @@ void search8(BYTE * * q_start,
 
                       if (score < UINT8_MAX)
                         {
-                          uint64_t offset = d_offset[c];
+                          const uint64_t offset = d_offset[c];
                           diff = backtrack_8(query.seq, dbseq, qlen, dbseqlen,
                                              dirbuffer,
                                              offset,
                                              dirbuffersize, c,
-                                             alignmentlengths + cand_id);
+                                             alignmentlengths + cand_id,
+                                             longestdbsequence);
                         }
                       else
                         {
@@ -1025,14 +988,14 @@ void search8(BYTE * * q_start,
 
                       diffs[cand_id] = diff;
 
-                      done++;
+                      ++done;
                     }
 
                   if (next_id < sequences)
                     {
                       // get next sequence
                       seq_id[c] = static_cast<int64_t>(next_id);
-                      uint64_t seqno = seqnos[next_id];
+                      const uint64_t seqno = seqnos[next_id];
                       char * address {nullptr};
                       unsigned int length {0};
 
@@ -1043,7 +1006,7 @@ void search8(BYTE * * q_start,
 
                       d_pos[c] = 0;
                       d_offset[c] = static_cast<uint64_t>(dir - dirbuffer);
-                      next_id++;
+                      ++next_id;
 
                       (reinterpret_cast<BYTE*>(&H0))[c] = 0;
                       (reinterpret_cast<BYTE*>(&F0))[c] = 2 * gap_open_penalty + 2 * gap_extend_penalty;
