@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,6 +23,11 @@
 
 
 #ifdef __x86_64__
+
+#ifdef __SSE2__
+#include <emmintrin.h>  // SSE2 intrinsics
+#endif
+
 #ifdef __SSE4_1__
 
 /*
@@ -36,8 +41,8 @@
   available starting with the Penryn architecture in 2008.
 */
 
-#include <cstdint>
-#include <smmintrin.h>
+#include <cstdint>  //uint64_t
+#include <smmintrin.h>  // _mm_min_epu16
 
 
 using VECTORTYPE = __m128i;
@@ -85,11 +90,11 @@ void align_cells_regular_16_sse41(VECTORTYPE * Sm,
                                   uint64_t * dir_long,
                                   VECTORTYPE * H0)
 {
-  constexpr unsigned int step {16};
-  constexpr unsigned int offset0 {0};
-  constexpr unsigned int offset1 {offset0 + 4};
-  constexpr unsigned int offset2 {offset1 + 4};
-  constexpr unsigned int offset3 {offset2 + 4};
+  static constexpr unsigned int step {16};
+  static constexpr unsigned int offset0 {0};
+  static constexpr unsigned int offset1 {offset0 + 4};
+  static constexpr unsigned int offset2 {offset1 + 4};
+  static constexpr unsigned int offset3 {offset2 + 4};
 
   VECTORTYPE Q;
   VECTORTYPE R;
@@ -168,11 +173,11 @@ void align_cells_masked_16_sse41(VECTORTYPE * Sm,
                                  VECTORTYPE * MR,
                                  VECTORTYPE * MQ0)
 {
-  constexpr unsigned int step {16};
-  constexpr unsigned int offset0 {0};
-  constexpr unsigned int offset1 {offset0 + 4};
-  constexpr unsigned int offset2 {offset1 + 4};
-  constexpr unsigned int offset3 {offset2 + 4};
+  static constexpr unsigned int step {16};
+  static constexpr unsigned int offset0 {0};
+  static constexpr unsigned int offset1 {offset0 + 4};
+  static constexpr unsigned int offset2 {offset1 + 4};
+  static constexpr unsigned int offset3 {offset2 + 4};
 
   VECTORTYPE Q;
   VECTORTYPE R;
