@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,6 +23,11 @@
 
 
 #ifdef __x86_64__
+
+#ifdef __SSE2__
+#include <emmintrin.h>  // SSE2 intrinsics
+#endif
+
 #ifdef __SSSE3__
 
 /*
@@ -36,7 +41,7 @@
   available starting with the Intel Core architecture in 2006.
 */
 
-#include <tmmintrin.h>
+#include <tmmintrin.h>  // _mm_shuffle_epi8
 
 #define CAST_m128i_ptr(x) (reinterpret_cast<__m128i*>(x))
 
@@ -91,7 +96,7 @@ void dprofile_shuffle16(WORD * dprofile,
                         WORD * score_matrix,
                         BYTE * dseq_byte)
 {
-  constexpr unsigned int channels {8};  // does 8 represent the number of channels?
+  static constexpr unsigned int channels {8};  // does 8 represent the number of channels?
   __m128i m0;
   __m128i m1;
   __m128i m2;
