@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,35 +23,42 @@
 
 #include "swarm.h"
 #include "utils/fatal.h"
+#include <cstdint>  // uint64_t
+#include <cstdio>  // fdopen, FILE, getline, size_t
+#include <cstdlib>  // free, posix_memalign, realloc
+#include <cstring>  // strcmp
+#include <sys/types.h>  // ssize_t
+#include <unistd.h>  // dup, STDIN_FILENO, STDOUT_FILENO
+
 
 static const char * progress_prompt;
 static uint64_t progress_next;
 static uint64_t progress_size;
 static uint64_t progress_chunk;
-constexpr size_t memalignment = 16;
+constexpr std::size_t memalignment = 16;
 
 
 auto progress_init(const char * prompt, const uint64_t size) -> void
 {
-  constexpr uint64_t progress_granularity {200};
+  static constexpr uint64_t progress_granularity {200};
   progress_prompt = prompt;
   progress_size = size;
   progress_chunk = size < progress_granularity ?
     1 : size / progress_granularity;
   progress_next = 1;
-  if (! opt_log.empty()) {
-    fprintf(logfile, "%s", prompt);
+  if (not opt_log.empty()) {
+    std::fprintf(logfile, "%s", prompt);
   }
   else {
-    fprintf(logfile, "%s %.0f%%", prompt, 0.0);
+    std::fprintf(logfile, "%s %.0f%%", prompt, 0.0);
   }
 }
 
 auto progress_update(const uint64_t progress) -> void
 {
-  if (opt_log.empty() && (progress >= progress_next))
+  if (opt_log.empty() and (progress >= progress_next))
     {
-      fprintf(logfile, "  \r%s %.0f%%", progress_prompt,
+      std::fprintf(logfile, "  \r%s %.0f%%", progress_prompt,
               100.0 * static_cast<double>(progress)
               / static_cast<double>(progress_size));
       progress_next = progress + progress_chunk;
@@ -61,17 +68,17 @@ auto progress_update(const uint64_t progress) -> void
 
 auto progress_done() -> void
 {
-  if (! opt_log.empty()) {
-    fprintf(logfile, " %.0f%%\n", 100.0);
+  if (not opt_log.empty()) {
+    std::fprintf(logfile, " %.0f%%\n", 100.0);
   }
   else {
-    fprintf(logfile, "  \r%s %.0f%%\n", progress_prompt, 100.0);
+    std::fprintf(logfile, "  \r%s %.0f%%\n", progress_prompt, 100.0);
   }
   fflush(logfile);
 }
 
 
-auto xmalloc(size_t size) -> void *
+auto xmalloc(std::size_t size) -> void *
 {
   if (size == 0) {
     size = 1;
@@ -91,7 +98,7 @@ auto xmalloc(size_t size) -> void *
 }
 
 
-auto xrealloc(void *ptr, size_t size) -> void *
+auto xrealloc(void *ptr, std::size_t size) -> void *
 {
   if (size == 0) {
     size = 1;
@@ -99,7 +106,7 @@ auto xrealloc(void *ptr, size_t size) -> void *
 #ifdef _WIN32
   void * t = _aligned_realloc(ptr, size, memalignment);
 #else
-  void * t = realloc(ptr, size);
+  void * t = std::realloc(ptr, size);
 #endif
   if (t == nullptr) {
     fatal(error_prefix, "Unable to allocate enough memory.");
@@ -114,7 +121,7 @@ auto xfree(void * ptr) -> void
 #ifdef _WIN32
       _aligned_free(ptr);
 #else
-      free(ptr);
+      std::free(ptr);
 #endif
     }
   else {
@@ -127,12 +134,12 @@ auto fopen_input(const char * filename) -> std::FILE *
   /* open the input stream given by filename, but use stdin if name is - */
   std::FILE * input_stream = nullptr;
 
-  if (strcmp(filename, "-") == 0) {
-    const int fd = dup(STDIN_FILENO);
-    input_stream = fd > 0 ? fdopen(fd, "rb") : nullptr;
+  if (std::strcmp(filename, "-") == 0) {
+    const int file_descriptor = dup(STDIN_FILENO);
+    input_stream = file_descriptor > 0 ? fdopen(file_descriptor, "rb") : nullptr;
   }
   else {
-    input_stream = fopen(filename, "rb");
+    input_stream = fopen(filename, "rb");  // refactoring: prefer std::fstream
   }
 
   return input_stream;
@@ -143,18 +150,18 @@ auto fopen_output(const char * filename) -> std::FILE *
   /* open the output stream given by filename, but use stdout if name is - */
   std::FILE * output_stream {nullptr};
 
-  if (strcmp(filename, "-") == 0) {
-    const int fd = dup(STDOUT_FILENO);
-    output_stream = fd > 0 ? fdopen(fd, "w") : nullptr;
+  if (std::strcmp(filename, "-") == 0) {
+    const int file_descriptor = dup(STDOUT_FILENO);
+    output_stream = file_descriptor > 0 ? fdopen(file_descriptor, "w") : nullptr;
   }
   else {
-    output_stream = fopen(filename, "w");
+    output_stream = fopen(filename, "w");  // refactoring: prefer std::fstream
   }
 
   return output_stream;
 }
 
-auto xgetline(char ** linep, size_t * linecapp, FILE * stream) -> ssize_t
+auto xgetline(char ** linep, std::size_t * linecapp, std::FILE * stream) -> ssize_t
 {
 #ifndef _WIN32
 
@@ -173,11 +180,11 @@ auto xgetline(char ** linep, size_t * linecapp, FILE * stream) -> ssize_t
      which is important for correct counting of characters and file size.
   */
 
-  const size_t minsize = 2;
-  const size_t maxsize = SIZE_MAX / 2;
+  const std::size_t minsize = 2;
+  const std::size_t maxsize = SIZE_MAX / 2;
 
   /* Error if linep or linecapp pointers are null */
-  if ((linep == nullptr) || (linecapp == nullptr))
+  if ((linep == nullptr) or (linecapp == nullptr))
     {
       errno = EINVAL;
       return -1;
@@ -194,7 +201,7 @@ auto xgetline(char ** linep, size_t * linecapp, FILE * stream) -> ssize_t
 
   char * p = *linep;            // pointer to where to put next char
   char * e = p + *linecapp - 1; // pointer to last byte in buffer
-  size_t len = 0;
+  std::size_t len = 0;
   *p = 0;
 
   while (1)
@@ -242,8 +249,8 @@ auto xgetline(char ** linep, size_t * linecapp, FILE * stream) -> ssize_t
           return -1;
         }
 
-      size_t newlinecap = minsize;
-      while ((newlinecap <= *linecapp) && (newlinecap < maxsize))
+      std::size_t newlinecap = minsize;
+      while ((newlinecap <= *linecapp) and (newlinecap < maxsize))
         newlinecap *= 2;
 
       if (newlinecap > maxsize)
@@ -252,7 +259,7 @@ auto xgetline(char ** linep, size_t * linecapp, FILE * stream) -> ssize_t
           return -1;
         }
 
-      char * newlinep = (char *) realloc(*linep, newlinecap);
+      char * newlinep = (char *) std::realloc(*linep, newlinecap);
       if (newlinep == nullptr)
         {
           // Memory allocation error
