@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,17 +23,31 @@
 
 #include <iostream>
 
-/* message and exit with an error (variadic template with compile-time recursion) */
-
-// C++17: refactor with fold expression
-// C++20: refactor with Printable concept
-
-static char const * const error_prefix {"\nError: "};  // C++17: refactor by moving to fatal template
 
+static char const * const error_prefix {"\nError: "};  // refactoring C++17: move into template
 auto fatal() -> void;
 
+
+/* message and exit with an error (variadic template with compile-time recursion) */
+
+// refactoring C++11: constrain with type traits?
+// refactoring C++17: use fold expression
+// refactoring C++20: use Printable concept
+
+// general case: zero or n arguments
+template<typename... T>
+auto fatal(T... args) -> void;
+
+// consume arguments one-by-one
 template <typename T, typename... Tail>
 auto fatal(T head, Tail... tail) -> void {
     std::cerr << head;
     fatal(tail...);
 }
+
+// // specialization: zero argument  // triggers a multiple definitions error at link time
+// template<>
+// auto fatal() -> void {
+//     std::cerr << '\n';
+//     std::exit(EXIT_FAILURE);
+// }
