@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2023 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,18 +21,17 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-/* Variant information */
+#include <cstdint>
 
-constexpr int substitution {1};
-constexpr int deletion {2};
-constexpr int insertion {3};
+/* Variant information */
+enum class Variant_type : unsigned char { substitution, deletion, insertion };
 
 struct var_s
 {
   uint64_t hash;
   unsigned int pos;
-  unsigned char type;
-  unsigned char base;
+  Variant_type type;
+  unsigned char base;  // encoded nucleotide: 0, 1, 2, or 3
   unsigned short dummy; /* for alignment padding only */
 };
 
